info = {
    "name": "jmc",
    "date_order": "DMY",
    "january": [
        "jan",
        "januari"
    ],
    "february": [
        "feb",
        "februari"
    ],
    "march": [
        "mac",
        "machi"
    ],
    "april": [
        "apr",
        "aprilyi"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "junyi"
    ],
    "july": [
        "jul",
        "julyai"
    ],
    "august": [
        "ago",
        "agusti"
    ],
    "september": [
        "sep",
        "septemba"
    ],
    "october": [
        "okt",
        "oktoba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "des",
        "desemba"
    ],
    "monday": [
        "jtt",
        "jumatatuu"
    ],
    "tuesday": [
        "jnn",
        "jumanne"
    ],
    "wednesday": [
        "jtn",
        "jumatanu"
    ],
    "thursday": [
        "alh",
        "alhamisi"
    ],
    "friday": [
        "iju",
        "ijumaa"
    ],
    "saturday": [
        "jmo",
        "jumamosi"
    ],
    "sunday": [
        "jpi",
        "jumapilyi"
    ],
    "am": [
        "utuko"
    ],
    "pm": [
        "kyiukonyi"
    ],
    "year": [
        "maka"
    ],
    "month": [
        "mori"
    ],
    "week": [
        "wiikyi"
    ],
    "day": [
        "mfiri"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "dakyika"
    ],
    "second": [
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "inu"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ukou"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ngama"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ar",
    "date_order": "DMY",
    "january": [
        "يناير"
    ],
    "february": [
        "فبراير"
    ],
    "march": [
        "مارس"
    ],
    "april": [
        "أبريل"
    ],
    "may": [
        "مايو"
    ],
    "june": [
        "يونيو"
    ],
    "july": [
        "يوليو"
    ],
    "august": [
        "أغسطس"
    ],
    "september": [
        "سبتمبر"
    ],
    "october": [
        "أكتوبر"
    ],
    "november": [
        "نوفمبر"
    ],
    "december": [
        "ديسمبر"
    ],
    "monday": [
        "الاثنين",
        "الإثنين"
    ],
    "tuesday": [
        "الثلاثاء"
    ],
    "wednesday": [
        "الأربعاء"
    ],
    "thursday": [
        "الخميس"
    ],
    "friday": [
        "الجمعة"
    ],
    "saturday": [
        "السبت"
    ],
    "sunday": [
        "الأحد"
    ],
    "am": [
        "ص",
        "صباحًا",
        "صباحاً"
    ],
    "pm": [
        "م",
        "مساءً"
    ],
    "year": [
        "السنة",
        "عام",
        "سنة"
    ],
    "month": [
        "الشهر",
        "شهر"
    ],
    "week": [
        "الأسبوع",
        "أسبوع"
    ],
    "day": [
        "يوم",
        "أيام"
    ],
    "hour": [
        "الساعات",
        "ساعة",
        "ساعات"
    ],
    "minute": [
        "الدقائق",
        "دقيقة",
        "دقائق"
    ],
    "second": [
        "الثواني",
        "ثانية"
    ],
    "relative-type": {
        "0 day ago": [
            "اليوم"
        ],
        "0 hour ago": [
            "الساعة الحالية"
        ],
        "0 minute ago": [
            "هذه الدقيقة"
        ],
        "0 month ago": [
            "هذا الشهر"
        ],
        "0 second ago": [
            "الآن"
        ],
        "0 week ago": [
            "هذا الأسبوع"
        ],
        "0 year ago": [
            "السنة الحالية"
        ],
        "1 day ago": [
            "أمس",
            "اليوم السابق",
            "الأمس"
        ],
        "1 month ago": [
            "الشهر الماضي"
        ],
        "1 week ago": [
            "الأسبوع الماضي"
        ],
        "1 year ago": [
            "السنة الماضية"
        ],
        "in 1 day": [
            "غدًا"
        ],
        "in 1 month": [
            "الشهر القادم"
        ],
        "in 1 week": [
            "الأسبوع القادم"
        ],
        "in 1 year": [
            "السنة القادمة"
        ],
        "2 day": [
            "يومين"
        ],
        "2 hour": [
            "ساعتين"
        ],
        "1 hour ago": [
            "ساعة واحدة"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "قبل (\\d+[.,]?\\d*) يوم"
        ],
        "\\1 hour ago": [
            "قبل (\\d+[.,]?\\d*) ساعة"
        ],
        "\\1 minute ago": [
            "قبل (\\d+[.,]?\\d*) دقيقة"
        ],
        "\\1 month ago": [
            "قبل (\\d+[.,]?\\d*) شهر"
        ],
        "\\1 second ago": [
            "قبل (\\d+[.,]?\\d*) ثانية"
        ],
        "\\1 week ago": [
            "قبل (\\d+[.,]?\\d*) أسبوع"
        ],
        "\\1 year ago": [
            "قبل (\\d+[.,]?\\d*) سنة"
        ],
        "in \\1 day": [
            "خلال (\\d+[.,]?\\d*) يوم"
        ],
        "in \\1 hour": [
            "خلال (\\d+[.,]?\\d*) ساعة"
        ],
        "in \\1 minute": [
            "خلال (\\d+[.,]?\\d*) دقيقة"
        ],
        "in \\1 month": [
            "خلال (\\d+[.,]?\\d*) شهر"
        ],
        "in \\1 second": [
            "خلال (\\d+[.,]?\\d*) ثانية"
        ],
        "in \\1 week": [
            "خلال (\\d+[.,]?\\d*) أسبوع"
        ],
        "in \\1 year": [
            "خلال (\\d+[.,]?\\d*) سنة"
        ]
    },
    "locale_specific": {
        "ar-AE": {
            "name": "ar-AE",
            "relative-type": {
                "0 year ago": [
                    "هذه السنة"
                ],
                "in 1 year": [
                    "السنة التالية"
                ]
            }
        },
        "ar-BH": {
            "name": "ar-BH"
        },
        "ar-DJ": {
            "name": "ar-DJ"
        },
        "ar-DZ": {
            "name": "ar-DZ",
            "january": [
                "جانفي"
            ],
            "february": [
                "فيفري"
            ],
            "april": [
                "أفريل"
            ],
            "may": [
                "ماي"
            ],
            "june": [
                "جوان"
            ],
            "july": [
                "جويلية"
            ],
            "august": [
                "أوت"
            ]
        },
        "ar-EG": {
            "name": "ar-EG"
        },
        "ar-EH": {
            "name": "ar-EH"
        },
        "ar-ER": {
            "name": "ar-ER"
        },
        "ar-IL": {
            "name": "ar-IL"
        },
        "ar-IQ": {
            "name": "ar-IQ",
            "january": [
                "كانون الثاني"
            ],
            "february": [
                "شباط"
            ],
            "march": [
                "آذار"
            ],
            "april": [
                "نيسان"
            ],
            "may": [
                "أيار"
            ],
            "june": [
                "حزيران"
            ],
            "july": [
                "تموز"
            ],
            "august": [
                "آب"
            ],
            "september": [
                "أيلول"
            ],
            "october": [
                "تشرين الأول",
                "تشرین الأول"
            ],
            "november": [
                "تشرين الثاني"
            ],
            "december": [
                "كانون الأول"
            ]
        },
        "ar-JO": {
            "name": "ar-JO",
            "january": [
                "كانون الثاني"
            ],
            "february": [
                "شباط"
            ],
            "march": [
                "آذار"
            ],
            "april": [
                "نيسان"
            ],
            "may": [
                "أيار"
            ],
            "june": [
                "حزيران"
            ],
            "july": [
                "تموز"
            ],
            "august": [
                "آب"
            ],
            "september": [
                "أيلول"
            ],
            "october": [
                "تشرين الأول"
            ],
            "november": [
                "تشرين الثاني"
            ],
            "december": [
                "كانون الأول"
            ]
        },
        "ar-KM": {
            "name": "ar-KM"
        },
        "ar-KW": {
            "name": "ar-KW"
        },
        "ar-LB": {
            "name": "ar-LB",
            "january": [
                "كانون الثاني"
            ],
            "february": [
                "شباط"
            ],
            "march": [
                "آذار"
            ],
            "april": [
                "نيسان"
            ],
            "may": [
                "أيار"
            ],
            "june": [
                "حزيران"
            ],
            "july": [
                "تموز"
            ],
            "august": [
                "آب"
            ],
            "september": [
                "أيلول"
            ],
            "october": [
                "تشرين الأول"
            ],
            "november": [
                "تشرين الثاني"
            ],
            "december": [
                "كانون الأول"
            ]
        },
        "ar-LY": {
            "name": "ar-LY"
        },
        "ar-MA": {
            "name": "ar-MA",
            "may": [
                "ماي"
            ],
            "july": [
                "يوليوز"
            ],
            "august": [
                "غشت"
            ],
            "september": [
                "شتنبر"
            ],
            "november": [
                "نونبر"
            ],
            "december": [
                "دجنبر"
            ]
        },
        "ar-MR": {
            "name": "ar-MR",
            "april": [
                "إبريل"
            ],
            "august": [
                "أغشت"
            ],
            "september": [
                "شتمبر"
            ],
            "december": [
                "دجمبر"
            ]
        },
        "ar-OM": {
            "name": "ar-OM"
        },
        "ar-PS": {
            "name": "ar-PS",
            "january": [
                "كانون الثاني"
            ],
            "february": [
                "شباط"
            ],
            "march": [
                "آذار"
            ],
            "april": [
                "نيسان"
            ],
            "may": [
                "أيار"
            ],
            "june": [
                "حزيران"
            ],
            "july": [
                "تموز"
            ],
            "august": [
                "آب"
            ],
            "september": [
                "أيلول"
            ],
            "october": [
                "تشرين الأول"
            ],
            "november": [
                "تشرين الثاني"
            ],
            "december": [
                "كانون الأول"
            ]
        },
        "ar-QA": {
            "name": "ar-QA"
        },
        "ar-SA": {
            "name": "ar-SA"
        },
        "ar-SD": {
            "name": "ar-SD"
        },
        "ar-SO": {
            "name": "ar-SO"
        },
        "ar-SS": {
            "name": "ar-SS"
        },
        "ar-SY": {
            "name": "ar-SY",
            "january": [
                "كانون الثاني"
            ],
            "february": [
                "شباط"
            ],
            "march": [
                "آذار"
            ],
            "april": [
                "نيسان"
            ],
            "may": [
                "أيار"
            ],
            "june": [
                "حزيران"
            ],
            "july": [
                "تموز"
            ],
            "august": [
                "آب"
            ],
            "september": [
                "أيلول"
            ],
            "october": [
                "تشرين الأول"
            ],
            "november": [
                "تشرين الثاني"
            ],
            "december": [
                "كانون الأول"
            ]
        },
        "ar-TD": {
            "name": "ar-TD"
        },
        "ar-TN": {
            "name": "ar-TN",
            "january": [
                "جانفي"
            ],
            "february": [
                "فيفري"
            ],
            "april": [
                "أفريل"
            ],
            "may": [
                "ماي"
            ],
            "june": [
                "جوان"
            ],
            "july": [
                "جويلية"
            ],
            "august": [
                "أوت"
            ]
        },
        "ar-YE": {
            "name": "ar-YE"
        }
    },
    "sentence_splitter_group": 6,
    "skip": [
        "الساعة",
        "في",
        "مساءً",
        "هـ",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "ago": [
        "منذ",
        "مضت"
    ],
    "in": [
        "خلال"
    ]
}



info = {
    "name": "gd",
    "date_order": "DMY",
    "january": [
        "am faoilleach",
        "dhen fhaoilleach",
        "faoi"
    ],
    "february": [
        "an gearran",
        "dhen ghearran",
        "gearr"
    ],
    "march": [
        "am màrt",
        "dhen mhàrt",
        "màrt"
    ],
    "april": [
        "an giblean",
        "dhen ghiblean",
        "gibl"
    ],
    "may": [
        "an cèitean",
        "cèit",
        "dhen chèitean"
    ],
    "june": [
        "an t-ògmhios",
        "dhen ògmhios",
        "ògmh"
    ],
    "july": [
        "an t-iuchar",
        "dhen iuchar",
        "iuch"
    ],
    "august": [
        "an lùnastal",
        "dhen lùnastal",
        "lùna"
    ],
    "september": [
        "an t-sultain",
        "dhen t-sultain",
        "sult"
    ],
    "october": [
        "an dàmhair",
        "dhen dàmhair",
        "dàmh"
    ],
    "november": [
        "an t-samhain",
        "dhen t-samhain",
        "samh"
    ],
    "december": [
        "an dùbhlachd",
        "dhen dùbhlachd",
        "dùbh"
    ],
    "monday": [
        "dil",
        "diluain"
    ],
    "tuesday": [
        "dim",
        "dimàirt"
    ],
    "wednesday": [
        "dic",
        "diciadain"
    ],
    "thursday": [
        "dia",
        "diardaoin"
    ],
    "friday": [
        "dih",
        "dihaoine"
    ],
    "saturday": [
        "dis",
        "disathairne"
    ],
    "sunday": [
        "did",
        "didòmhnaich"
    ],
    "am": [
        "m"
    ],
    "pm": [
        "f"
    ],
    "year": [
        "bl",
        "blia",
        "bliadhna"
    ],
    "month": [
        "mì",
        "mìos"
    ],
    "week": [
        "seachd",
        "seachdain",
        "sn"
    ],
    "day": [
        "latha",
        "là"
    ],
    "hour": [
        "u",
        "uair",
        "uair a thìde"
    ],
    "minute": [
        "m",
        "mion",
        "mionaid"
    ],
    "second": [
        "d",
        "diog"
    ],
    "relative-type": {
        "0 day ago": [
            "an-diugh"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "am mì seo",
            "am mìos seo"
        ],
        "0 second ago": [
            "an-dràsta"
        ],
        "0 week ago": [
            "an t-seachd seo",
            "an t-seachdain seo",
            "an t-sn seo"
        ],
        "0 year ago": [
            "am bl",
            "am bliadhna"
        ],
        "1 day ago": [
            "an-dè"
        ],
        "1 month ago": [
            "am mìos sa chaidh",
            "am mìos seo chaidh",
            "mì ch"
        ],
        "1 week ago": [
            "an t-seachdain seo chaidh",
            "seachd sa chaidh",
            "sn ch"
        ],
        "1 year ago": [
            "an-uir",
            "an-uiridh"
        ],
        "in 1 day": [
            "a-màireach"
        ],
        "in 1 month": [
            "an ath-mhìos",
            "ath-mhì"
        ],
        "in 1 week": [
            "an ath-sheachd",
            "an ath-sheachdain",
            "ath-shn"
        ],
        "in 1 year": [
            "an ath-bhl",
            "an ath-bhliadhna"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "o (\\d+[.,]?\\d*) là",
            "(\\d+[.,]?\\d*) latha air ais"
        ],
        "\\1 hour ago": [
            "o (\\d+[.,]?\\d*) uair",
            "(\\d+[.,]?\\d*) uair a thìde air ais"
        ],
        "\\1 minute ago": [
            "o (\\d+[.,]?\\d*) mhion",
            "o (\\d+[.,]?\\d*) mion",
            "(\\d+[.,]?\\d*) mhionaid air ais",
            "(\\d+[.,]?\\d*) mionaid air ais"
        ],
        "\\1 month ago": [
            "o (\\d+[.,]?\\d*) mhìos",
            "o (\\d+[.,]?\\d*) mìos",
            "(\\d+[.,]?\\d*) mhìos air ais",
            "(\\d+[.,]?\\d*) mìos air ais"
        ],
        "\\1 second ago": [
            "o (\\d+[.,]?\\d*) diog",
            "(\\d+[.,]?\\d*) diog air ais"
        ],
        "\\1 week ago": [
            "o (\\d+[.,]?\\d*) seachd",
            "o (\\d+[.,]?\\d*) sheachd",
            "(\\d+[.,]?\\d*) seachdain air ais"
        ],
        "\\1 year ago": [
            "o (\\d+[.,]?\\d*) bhlia",
            "o (\\d+[.,]?\\d*) blia",
            "(\\d+[.,]?\\d*) bhliadhna air ais",
            "(\\d+[.,]?\\d*) bliadhna air ais"
        ],
        "in \\1 day": [
            "an ceann (\\d+[.,]?\\d*) latha",
            "an (\\d+[.,]?\\d*) là"
        ],
        "in \\1 hour": [
            "an ceann (\\d+[.,]?\\d*) uair a thìde",
            "an (\\d+[.,]?\\d*) uair"
        ],
        "in \\1 minute": [
            "an ceann (\\d+[.,]?\\d*) mhionaid",
            "an ceann (\\d+[.,]?\\d*) mionaid",
            "an (\\d+[.,]?\\d*) mhion",
            "an (\\d+[.,]?\\d*) mion"
        ],
        "in \\1 month": [
            "an ceann (\\d+[.,]?\\d*) mhìosa",
            "an ceann (\\d+[.,]?\\d*) mìosa",
            "an (\\d+[.,]?\\d*) mhìos",
            "an (\\d+[.,]?\\d*) mìos"
        ],
        "in \\1 second": [
            "an ceann (\\d+[.,]?\\d*) diog",
            "an (\\d+[.,]?\\d*) diog"
        ],
        "in \\1 week": [
            "an ceann (\\d+[.,]?\\d*) seachdain",
            "an (\\d+[.,]?\\d*) seachd",
            "an (\\d+[.,]?\\d*) sheachd"
        ],
        "in \\1 year": [
            "an ceann (\\d+[.,]?\\d*) bhliadhna",
            "an ceann (\\d+[.,]?\\d*) bliadhna",
            "an (\\d+[.,]?\\d*) bhlia",
            "an (\\d+[.,]?\\d*) blia"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "uk",
    "date_order": "DMY",
    "january": [
        "січ",
        "січень",
        "січня"
    ],
    "february": [
        "лют",
        "лютий",
        "лютого"
    ],
    "march": [
        "бер",
        "березень",
        "березня",
        "берез"
    ],
    "april": [
        "кві",
        "квіт",
        "квітень",
        "квітня"
    ],
    "may": [
        "тра",
        "трав",
        "травень",
        "травня"
    ],
    "june": [
        "чер",
        "черв",
        "червень",
        "червня"
    ],
    "july": [
        "лип",
        "липень",
        "липня"
    ],
    "august": [
        "сер",
        "серп",
        "серпень",
        "серпня"
    ],
    "september": [
        "вер",
        "вересень",
        "вересня",
        "верес"
    ],
    "october": [
        "жов",
        "жовт",
        "жовтень",
        "жовтня"
    ],
    "november": [
        "лис",
        "лист",
        "листопад",
        "листопада",
        "листоп"
    ],
    "december": [
        "гру",
        "груд",
        "грудень",
        "грудня"
    ],
    "monday": [
        "пн",
        "понеділок",
        "пн",
        "пон",
        "понед",
        "понеділок"
    ],
    "tuesday": [
        "вт",
        "вівторок",
        "вт",
        "вів",
        "вівтор",
        "вівторок"
    ],
    "wednesday": [
        "середа",
        "ср",
        "ср",
        "сер",
        "серед",
        "середа",
        "середу"
    ],
    "thursday": [
        "четвер",
        "чт",
        "чт",
        "чет",
        "четвер"
    ],
    "friday": [
        "п'ятниця",
        "пт",
        "пят"
    ],
    "saturday": [
        "сб",
        "субота",
        "сб",
        "суб",
        "субот",
        "субота",
        "суботу"
    ],
    "sunday": [
        "нд",
        "неділя",
        "нд",
        "нед",
        "неділ",
        "неділя",
        "неділю"
    ],
    "am": [
        "дп",
        "до полудня"
    ],
    "pm": [
        "пп",
        "після полудня",
        "пополудні",
        "пополудні"
    ],
    "year": [
        "р",
        "рік",
        "р",
        "рік",
        "роки",
        "років",
        "року",
        "роком",
        "році",
        "роках",
        "роками",
        "рокові"
    ],
    "month": [
        "міс",
        "місяць",
        "м",
        "міс",
        "місяць",
        "місяці",
        "місяців",
        "місяця",
        "місяцю",
        "місяцям",
        "місяцем",
        "місяцеві",
        "місяцях",
        "місяцями"
    ],
    "week": [
        "тиж",
        "тиждень",
        "т",
        "тиждень",
        "тижня",
        "тижні",
        "тижнів",
        "тижнем",
        "тижневі",
        "тижню"
    ],
    "day": [
        "д",
        "день",
        "д",
        "дн",
        "день",
        "дня",
        "дні",
        "днів",
        "доба",
        "доби",
        "діб",
        "дневі",
        "дню",
        "днем"
    ],
    "hour": [
        "год",
        "година",
        "г",
        "год",
        "годин",
        "година",
        "години",
        "годин",
        "годину",
        "г",
        "годині",
        "годиною",
        "годино"
    ],
    "minute": [
        "хв",
        "хвилина",
        "хв",
        "хвилини",
        "хвилин",
        "хвилину",
        "хвилина",
        "хвилині",
        "хвилиною",
        "хвилино"
    ],
    "second": [
        "с",
        "секунда",
        "с",
        "секунда",
        "секунди",
        "секунд",
        "секунду",
        "сек",
        "секунді",
        "секундою",
        "секундо"
    ],
    "relative-type": {
        "0 day ago": [
            "сьогодні"
        ],
        "0 hour ago": [
            "цієї години"
        ],
        "0 minute ago": [
            "цієї хвилини"
        ],
        "0 month ago": [
            "цього місяця"
        ],
        "0 second ago": [
            "зараз"
        ],
        "0 week ago": [
            "цього тижня"
        ],
        "0 year ago": [
            "цього року"
        ],
        "1 day ago": [
            "учора",
            "вчора"
        ],
        "1 month ago": [
            "минулого місяця"
        ],
        "1 week ago": [
            "минулого тижня"
        ],
        "1 year ago": [
            "торік"
        ],
        "in 1 day": [
            "завтра"
        ],
        "in 1 month": [
            "наступного місяця"
        ],
        "in 1 week": [
            "наступного тижня"
        ],
        "in 1 year": [
            "наступного року"
        ],
        "2 day ago": [
            "позавчора"
        ],
        "in 2 day": [
            "післязавтра"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) д тому",
            "(\\d+[.,]?\\d*) день тому",
            "(\\d+[.,]?\\d*) дн тому",
            "(\\d+[.,]?\\d*) дня тому"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) год тому",
            "(\\d+[.,]?\\d*) години тому",
            "(\\d+[.,]?\\d*) годину тому",
            "(\\d+[.,]?\\d*) година тому"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) хв тому",
            "(\\d+[.,]?\\d*) хвилини тому",
            "(\\d+[.,]?\\d*) хвилину тому",
            "(\\d+[.,]?\\d*) хвилина тому"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) міс тому",
            "(\\d+[.,]?\\d*) місяць тому",
            "(\\d+[.,]?\\d*) місяця тому"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) с тому",
            "(\\d+[.,]?\\d*) секунди тому",
            "(\\d+[.,]?\\d*) секунду тому",
            "(\\d+[.,]?\\d*) секунда тому"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) тиж тому",
            "(\\d+[.,]?\\d*) тиждень тому",
            "(\\d+[.,]?\\d*) тижня тому"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) р тому",
            "(\\d+[.,]?\\d*) року тому",
            "(\\d+[.,]?\\d*) рік тому"
        ],
        "in \\1 day": [
            "за (\\d+[.,]?\\d*) д",
            "через (\\d+[.,]?\\d*) день",
            "через (\\d+[.,]?\\d*) дн",
            "через (\\d+[.,]?\\d*) дня"
        ],
        "in \\1 hour": [
            "за (\\d+[.,]?\\d*) год",
            "через (\\d+[.,]?\\d*) год",
            "через (\\d+[.,]?\\d*) години",
            "через (\\d+[.,]?\\d*) годину"
        ],
        "in \\1 minute": [
            "за (\\d+[.,]?\\d*) хв",
            "через (\\d+[.,]?\\d*) хв",
            "через (\\d+[.,]?\\d*) хвилини",
            "через (\\d+[.,]?\\d*) хвилину"
        ],
        "in \\1 month": [
            "за (\\d+[.,]?\\d*) міс",
            "через (\\d+[.,]?\\d*) міс",
            "через (\\d+[.,]?\\d*) місяць",
            "через (\\d+[.,]?\\d*) місяця"
        ],
        "in \\1 second": [
            "за (\\d+[.,]?\\d*) с",
            "через (\\d+[.,]?\\d*) с",
            "через (\\d+[.,]?\\d*) секунди",
            "через (\\d+[.,]?\\d*) секунду"
        ],
        "in \\1 week": [
            "за (\\d+[.,]?\\d*) тиж",
            "через (\\d+[.,]?\\d*) тиж",
            "через (\\d+[.,]?\\d*) тиждень",
            "через (\\d+[.,]?\\d*) тижня"
        ],
        "in \\1 year": [
            "за (\\d+[.,]?\\d*) р",
            "через (\\d+[.,]?\\d*) р",
            "через (\\d+[.,]?\\d*) року",
            "через (\\d+[.,]?\\d*) рік"
        ]
    },
    "locale_specific": {},
    "skip": [
        "приблизно",
        "близько",
        "о",
        "об",
        "і",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "тому",
        "назад"
    ],
    "in": [
        "протягом",
        "через",
        "за"
    ],
    "simplifications": [
        {
            "од(на|ну|ні|ної|нієї|ин|ній)": "1"
        },
        {
            "дв(а|і|оє|ох)": "2"
        },
        {
            "пар[ауі]": "2"
        },
        {
            "три": "3"
        },
        {
            "чотири": "4"
        },
        {
            "п'ять": "5"
        },
        {
            "шість": "6"
        },
        {
            "сім": "7"
        },
        {
            "вісім": "8"
        },
        {
            "дев'ять": "9"
        },
        {
            "десять": "10"
        },
        {
            "одинадцять": "11"
        },
        {
            "дванадцять": "12"
        },
        {
            "п'ятнадцять": "15"
        },
        {
            "двадцять": "20"
        },
        {
            "тридцять": "30"
        },
        {
            "сорок": "40"
        },
        {
            "п'ятдесят": "50"
        },
        {
            "півгодини": "30 хвилин"
        },
        {
            "пів години": "30 хвилин"
        },
        {
            "півроку": "6 місяців"
        },
        {
            "пів року": "6 місяців"
        },
        {
            "півтори години": "90 хвилин"
        },
        {
            "півтора року": "18 місяців"
        },
        {
            "^година": "1 година"
        },
        {
            "^годину": "1 годину"
        },
        {
            "^хвилину": "1 хвилину"
        },
        {
            "^секунду": "1 секунду"
        },
        {
            "кілька секунд": "44 секунди"
        },
        {
            "кілька хвилин": "2 хвилини"
        },
        {
            "опів на": "30 хвилин"
        },
        {
            "за чверть": "15 хвилин"
        }
    ]
}



from itertools import chain, zip_longest
from operator import methodcaller

import regex as re

from dateparser.utils import normalize_unicode

PARSER_HARDCODED_TOKENS = [":", ".", " ", "-", "/"]
PARSER_KNOWN_TOKENS = ["am", "pm", "UTC", "GMT", "Z"]
ALWAYS_KEEP_TOKENS = ["+"] + PARSER_HARDCODED_TOKENS
KNOWN_WORD_TOKENS = [
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday",
    "saturday",
    "sunday",
    "january",
    "february",
    "march",
    "april",
    "may",
    "june",
    "july",
    "august",
    "september",
    "october",
    "november",
    "december",
    "decade",
    "year",
    "month",
    "week",
    "day",
    "hour",
    "minute",
    "second",
    "ago",
    "in",
    "am",
    "pm",
]

PARENTHESES_PATTERN = re.compile(r"[\(\)]")
NUMERAL_PATTERN = re.compile(r"(\d+)")
KEEP_TOKEN_PATTERN = re.compile(r"^.*[^\W_].*$", flags=re.U)


class UnknownTokenError(Exception):
    pass


def _parse_bool(value):
    if isinstance(value, bool):
        return value
    if isinstance(value, str):
        return value.strip().lower() in ("true", "1", "yes")
    return False


class Dictionary:
    """
    Class that modifies and stores translations and handles splitting of date string.

    :param locale_info:
        Locale info (translation data) of the locale.
    :type language_info: dict

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :return: a Dictionary instance.
    """

    _split_regex_cache = {}
    _sorted_words_cache = {}
    _split_relative_regex_cache = {}
    _sorted_relative_strings_cache = {}
    _match_relative_regex_cache = {}

    def __init__(self, locale_info, settings=None):
        dictionary = {}
        self._settings = settings
        self.info = locale_info

        if "skip" in locale_info:
            skip = map(methodcaller("lower"), locale_info["skip"])
            dictionary.update(zip_longest(skip, [], fillvalue=None))
        if "pertain" in locale_info:
            pertain = map(methodcaller("lower"), locale_info["pertain"])
            dictionary.update(zip_longest(pertain, [], fillvalue=None))
        for word in KNOWN_WORD_TOKENS:
            if word in locale_info:
                translations = map(methodcaller("lower"), locale_info[word])
                dictionary.update(zip_longest(translations, [], fillvalue=word))
        dictionary.update(zip_longest(ALWAYS_KEEP_TOKENS, ALWAYS_KEEP_TOKENS))
        dictionary.update(
            zip_longest(
                map(methodcaller("lower"), PARSER_KNOWN_TOKENS), PARSER_KNOWN_TOKENS
            )
        )

        relative_type = locale_info.get("relative-type", {})
        for key, value in relative_type.items():
            relative_translations = map(methodcaller("lower"), value)
            dictionary.update(zip_longest(relative_translations, [], fillvalue=key))

        self._dictionary = dictionary

        no_word_spacing = locale_info.get("no_word_spacing", False)
        self._no_word_spacing = _parse_bool(no_word_spacing)

        relative_type_regex = locale_info.get("relative-type-regex", {})
        self._relative_strings = list(chain.from_iterable(relative_type_regex.values()))

    def __contains__(self, key):
        if key in self._settings.SKIP_TOKENS:
            return True
        return self._dictionary.__contains__(key)

    def __getitem__(self, key):
        if key in self._settings.SKIP_TOKENS:
            return None
        return self._dictionary.__getitem__(key)

    def __iter__(self):
        return chain(self._settings.SKIP_TOKENS, iter(self._dictionary))

    def are_tokens_valid(self, tokens):
        """
        Check if tokens are valid tokens for the locale.

        :param tokens:
            a list of string tokens.
        :type tokens: list

        :return: True if tokens are valid, False otherwise.
        """
        has_only_keep_tokens = not set(tokens) - set(ALWAYS_KEEP_TOKENS)
        if has_only_keep_tokens:
            return False
        match_relative_regex = self._get_match_relative_regex_cache()
        for token in tokens:
            if token.isdigit() or match_relative_regex.match(token) or token in self:
                continue
            else:
                return False
        else:
            return True

    def split(self, string, keep_formatting=False):
        """
        Split the date string using translations in locale info.

        :param string:
            Date string to be splitted.
        :type string:
            str

        :param keep_formatting:
            If True, retain formatting of the date string.
        :type keep_formatting: bool

        :return: A list of string tokens formed after splitting the date string.
        """
        if not string:
            return string

        split_relative_regex = self._get_split_relative_regex_cache()
        match_relative_regex = self._get_match_relative_regex_cache()

        tokens = split_relative_regex.split(string)

        for i, token in enumerate(tokens):
            if match_relative_regex.match(token):
                tokens[i] = [token]
                continue
            tokens[i] = self._split_by_known_words(token, keep_formatting)

        return list(filter(bool, chain.from_iterable(tokens)))

    def _add_to_cache(self, value, cache):
        cache.setdefault(self._settings.registry_key, {})[self.info["name"]] = value
        if (
            self._settings.CACHE_SIZE_LIMIT
            and len(cache) > self._settings.CACHE_SIZE_LIMIT
        ):
            cache.pop(list(cache.keys())[0])

    def _split_by_known_words(self, string: str, keep_formatting: bool):
        regex = self._get_split_regex_cache()
        splitted = []
        unknown = string

        while unknown:
            match = regex.match(string)

            if not match:
                curr_split = (
                    self._split_by_numerals(string, keep_formatting)
                    if self._should_capture(string, keep_formatting)
                    else []
                )
                unknown = ""
            else:
                unparsed, known, unknown = match.groups()
                curr_split = (
                    [known] if self._should_capture(known, keep_formatting) else []
                )

                if unparsed and self._should_capture(unparsed, keep_formatting):
                    curr_split = (
                        self._split_by_numerals(unparsed, keep_formatting) + curr_split
                    )

                if unknown:
                    string = unknown if string != unknown else ""

            for token in curr_split:
                if (
                    splitted
                    and splitted[-1].isdigit()
                    and token in {"st", "nd", "rd", "th"}
                ):
                    continue

                splitted.append(token)

        return splitted

    def _split_by_numerals(self, string, keep_formatting):
        return [
            token
            for token in NUMERAL_PATTERN.split(string)
            if self._should_capture(token, keep_formatting)
        ]

    def _should_capture(self, token, keep_formatting):
        return (
            keep_formatting
            or token in ALWAYS_KEEP_TOKENS
            or KEEP_TOKEN_PATTERN.match(token)
        )

    def _get_sorted_words_from_cache(self):
        if (
            self._settings.registry_key not in self._sorted_words_cache
            or self.info["name"]
            not in self._sorted_words_cache[self._settings.registry_key]
        ):
            self._add_to_cache(
                cache=self._sorted_words_cache,
                value=sorted([key for key in self], key=len, reverse=True),
            )
        return self._sorted_words_cache[self._settings.registry_key][self.info["name"]]

    def _get_split_regex_cache(self):
        if (
            self._settings.registry_key not in self._split_regex_cache
            or self.info["name"]
            not in self._split_regex_cache[self._settings.registry_key]
        ):
            self._construct_split_regex()
        return self._split_regex_cache[self._settings.registry_key][self.info["name"]]

    def _construct_split_regex(self):
        known_words_group = "|".join(
            map(re.escape, self._get_sorted_words_from_cache())
        )
        if self._no_word_spacing:
            regex = r"^(.*?)({})(.*)$".format(known_words_group)
        else:
            regex = r"^(.*?(?:\A|\W|_|\d))({})((?:\Z|\W|_|\d).*)$".format(
                known_words_group
            )
        self._add_to_cache(
            cache=self._split_regex_cache,
            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
        )

    def _get_sorted_relative_strings_from_cache(self):
        if (
            self._settings.registry_key not in self._sorted_relative_strings_cache
            or self.info["name"]
            not in self._sorted_relative_strings_cache[self._settings.registry_key]
        ):
            self._add_to_cache(
                cache=self._sorted_relative_strings_cache,
                value=sorted(
                    [
                        PARENTHESES_PATTERN.sub("", key)
                        for key in self._relative_strings
                    ],
                    key=len,
                    reverse=True,
                ),
            )
        return self._sorted_relative_strings_cache[self._settings.registry_key][
            self.info["name"]
        ]

    def _get_split_relative_regex_cache(self):
        if (
            self._settings.registry_key not in self._split_relative_regex_cache
            or self.info["name"]
            not in self._split_relative_regex_cache[self._settings.registry_key]
        ):
            self._construct_split_relative_regex()
        return self._split_relative_regex_cache[self._settings.registry_key][
            self.info["name"]
        ]

    def _construct_split_relative_regex(self):
        known_relative_strings_group = "|".join(
            self._get_sorted_relative_strings_from_cache()
        )
        if self._no_word_spacing:
            regex = "({})".format(known_relative_strings_group)
        else:
            regex = "(?<=(?:\\A|\\W|_))({})(?=(?:\\Z|\\W|_))".format(
                known_relative_strings_group
            )
        self._add_to_cache(
            cache=self._split_relative_regex_cache,
            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
        )

    def _get_match_relative_regex_cache(self):
        if (
            self._settings.registry_key not in self._match_relative_regex_cache
            or self.info["name"]
            not in self._match_relative_regex_cache[self._settings.registry_key]
        ):
            self._construct_match_relative_regex()
        return self._match_relative_regex_cache[self._settings.registry_key][
            self.info["name"]
        ]

    def _construct_match_relative_regex(self):
        known_relative_strings_group = "|".join(
            self._get_sorted_relative_strings_from_cache()
        )
        regex = "^({})$".format(known_relative_strings_group)
        self._add_to_cache(
            cache=self._match_relative_regex_cache,
            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
        )


class NormalizedDictionary(Dictionary):
    def __init__(self, locale_info, settings=None):
        super().__init__(locale_info, settings)
        self._normalize()

    def _normalize(self):
        new_dict = {}
        conflicting_keys = []
        for key, value in self._dictionary.items():
            normalized = normalize_unicode(key)
            if key != normalized and normalized in self._dictionary:
                conflicting_keys.append(key)
            else:
                new_dict[normalized] = value
        for key in conflicting_keys:
            normalized = normalize_unicode(key)
            if key in (self.info.get("skip", []) + self.info.get("pertain", [])):
                new_dict[normalized] = self._dictionary[key]
        self._dictionary = new_dict
        self._relative_strings = list(map(normalize_unicode, self._relative_strings))



info = {
    "name": "ca",
    "date_order": "DMY",
    "january": [
        "de gen",
        "de gener",
        "gen",
        "gener"
    ],
    "february": [
        "de febr",
        "de febrer",
        "febr",
        "febrer"
    ],
    "march": [
        "de març",
        "març"
    ],
    "april": [
        "abr",
        "abril",
        "d'abr",
        "d'abril"
    ],
    "may": [
        "de maig",
        "maig"
    ],
    "june": [
        "de juny",
        "juny"
    ],
    "july": [
        "de jul",
        "de juliol",
        "jul",
        "juliol"
    ],
    "august": [
        "ag",
        "agost",
        "d'ag",
        "d'agost"
    ],
    "september": [
        "de set",
        "de setembre",
        "set",
        "setembre"
    ],
    "october": [
        "d'oct",
        "d'octubre",
        "oct",
        "octubre"
    ],
    "november": [
        "de nov",
        "de novembre",
        "nov",
        "novembre"
    ],
    "december": [
        "de des",
        "de desembre",
        "des",
        "desembre"
    ],
    "monday": [
        "dilluns",
        "dl"
    ],
    "tuesday": [
        "dimarts",
        "dt"
    ],
    "wednesday": [
        "dc",
        "dimecres"
    ],
    "thursday": [
        "dijous",
        "dj"
    ],
    "friday": [
        "divendres",
        "dv"
    ],
    "saturday": [
        "dissabte",
        "ds"
    ],
    "sunday": [
        "dg",
        "diumenge"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "any"
    ],
    "month": [
        "mes"
    ],
    "week": [
        "setm",
        "setmana"
    ],
    "day": [
        "dia"
    ],
    "hour": [
        "h",
        "hora"
    ],
    "minute": [
        "min",
        "minut"
    ],
    "second": [
        "s",
        "segon"
    ],
    "relative-type": {
        "0 day ago": [
            "avui",
            "hui"
        ],
        "0 hour ago": [
            "aquesta hora"
        ],
        "0 minute ago": [
            "aquest minut"
        ],
        "0 month ago": [
            "aquest mes"
        ],
        "0 second ago": [
            "ara"
        ],
        "0 week ago": [
            "aquesta setm",
            "aquesta setmana"
        ],
        "0 year ago": [
            "enguany"
        ],
        "1 day ago": [
            "ahir"
        ],
        "1 month ago": [
            "el mes passat",
            "mes passat"
        ],
        "1 week ago": [
            "la setm passada",
            "la setmana passada",
            "setm passada"
        ],
        "1 year ago": [
            "l'any passat"
        ],
        "in 1 day": [
            "demà"
        ],
        "in 1 month": [
            "el mes que ve",
            "mes vinent"
        ],
        "in 1 week": [
            "la setm que ve",
            "la setmana que ve",
            "setm vinent",
            "la setmana vinent",
            "la pròxima setmana",
            "la propera setmana"
        ],
        "in 1 year": [
            "l'any que ve"
        ],
        "2 day ago": [
            "despús-ahir",
            "abans-d’ahir",
            "dellà-ahir"
        ],
        "in 2 day": [
            "endemà",
            "sendemà",
            "despús-demà",
            "demà passat",
            "passat demà"
        ],
        "in 3 day": [
            "endemà passat"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "fa (\\d+[.,]?\\d*) dia",
            "fa (\\d+[.,]?\\d*) dies"
        ],
        "\\1 hour ago": [
            "fa (\\d+[.,]?\\d*) h",
            "fa (\\d+[.,]?\\d*) hora",
            "fa (\\d+[.,]?\\d*) hores"
        ],
        "\\1 minute ago": [
            "fa (\\d+[.,]?\\d*) min",
            "fa (\\d+[.,]?\\d*) minut",
            "fa (\\d+[.,]?\\d*) minuts"
        ],
        "\\1 month ago": [
            "fa (\\d+[.,]?\\d*) mes",
            "fa (\\d+[.,]?\\d*) mesos"
        ],
        "\\1 second ago": [
            "fa (\\d+[.,]?\\d*) s",
            "fa (\\d+[.,]?\\d*) segon",
            "fa (\\d+[.,]?\\d*) segons"
        ],
        "\\1 week ago": [
            "fa (\\d+[.,]?\\d*) setm",
            "fa (\\d+[.,]?\\d*) setmana",
            "fa (\\d+[.,]?\\d*) setmanes"
        ],
        "\\1 year ago": [
            "fa (\\d+[.,]?\\d*) any",
            "fa (\\d+[.,]?\\d*) anys"
        ],
        "in \\1 day": [
            "d'aquí a (\\d+[.,]?\\d*) dia",
            "d'aquí a (\\d+[.,]?\\d*) dies"
        ],
        "in \\1 hour": [
            "d'aquí a (\\d+[.,]?\\d*) h",
            "d'aquí a (\\d+[.,]?\\d*) hora",
            "d'aquí a (\\d+[.,]?\\d*) hores",
            "d‘aquí a (\\d+[.,]?\\d*) h"
        ],
        "in \\1 minute": [
            "d'aquí a (\\d+[.,]?\\d*) min",
            "d'aquí a (\\d+[.,]?\\d*) minut",
            "d'aquí a (\\d+[.,]?\\d*) minuts"
        ],
        "in \\1 month": [
            "d'aquí a (\\d+[.,]?\\d*) mes",
            "d'aquí a (\\d+[.,]?\\d*) mesos"
        ],
        "in \\1 second": [
            "d'aquí a (\\d+[.,]?\\d*) s",
            "d'aquí a (\\d+[.,]?\\d*) segon",
            "d'aquí a (\\d+[.,]?\\d*) segons"
        ],
        "in \\1 week": [
            "d'aquí a (\\d+[.,]?\\d*) setm",
            "d'aquí a (\\d+[.,]?\\d*) setmana",
            "d'aquí a (\\d+[.,]?\\d*) setmanes"
        ],
        "in \\1 year": [
            "d'aquí a (\\d+[.,]?\\d*) any",
            "d'aquí a (\\d+[.,]?\\d*) anys"
        ]
    },
    "locale_specific": {
        "ca-AD": {
            "name": "ca-AD"
        },
        "ca-FR": {
            "name": "ca-FR"
        },
        "ca-IT": {
            "name": "ca-IT"
        }
    },
    "skip": [
        "de",
        "del",
        "i",
        "l'",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "pertain": [
        "de",
        "del"
    ],
    "in": [
        "en"
    ],
    "simplifications": [
        {
            "una": "1"
        },
        {
            "un": "1"
        }
    ]
}



from datetime import datetime, timedelta, timezone, tzinfo

import regex as re

from .timezones import timezone_info_list


class StaticTzInfo(tzinfo):
    def __init__(self, name, offset):
        self.__offset = offset
        self.__name = name

    def tzname(self, dt):
        return self.__name

    def utcoffset(self, dt):
        return self.__offset

    def dst(self, dt):
        return timedelta(0)

    def __repr__(self):
        return "<%s '%s'>" % (self.__class__.__name__, self.__name)

    def localize(self, dt, is_dst=False):
        if dt.tzinfo is not None:
            raise ValueError("Not naive datetime (tzinfo is already set)")
        return dt.replace(tzinfo=self)

    def __getinitargs__(self):
        return self.__name, self.__offset


def pop_tz_offset_from_string(date_string, as_offset=True):
    if _search_regex_ignorecase.search(date_string):
        for name, info in _tz_offsets:
            timezone_re = info["regex"]
            timezone_match = timezone_re.search(date_string)
            if timezone_match:
                start, stop = timezone_match.span()
                date_string = date_string[: start + 1] + date_string[stop:]
                return (
                    date_string,
                    StaticTzInfo(name, info["offset"]) if as_offset else name,
                )
    return date_string, None


def word_is_tz(word):
    return bool(_search_regex.match(word))


def convert_to_local_tz(datetime_obj, datetime_tz_offset):
    return datetime_obj - datetime_tz_offset + local_tz_offset


def build_tz_offsets(search_regex_parts):
    def get_offset(tz_obj, regex, repl="", replw=""):
        return (
            tz_obj[0],
            {
                "regex": re.compile(
                    re.sub(repl, replw, regex % tz_obj[0]), re.IGNORECASE
                ),
                "offset": timedelta(seconds=tz_obj[1]),
            },
        )

    for tz_info in timezone_info_list:
        for regex in tz_info["regex_patterns"]:
            for tz_obj in tz_info["timezones"]:
                search_regex_parts.append(tz_obj[0])
                yield get_offset(tz_obj, regex)

                # alternate patterns
                for replace, replacewith in tz_info.get("replace", []):
                    search_regex_parts.append(re.sub(replace, replacewith, tz_obj[0]))
                    yield get_offset(tz_obj, regex, repl=replace, replw=replacewith)


def get_local_tz_offset():
    offset = datetime.now() - datetime.now(tz=timezone.utc).replace(tzinfo=None)
    offset = timedelta(days=offset.days, seconds=round(offset.seconds, -1))
    return offset


_search_regex_parts = []
_tz_offsets = list(build_tz_offsets(_search_regex_parts))
_search_regex = re.compile("|".join(_search_regex_parts))
_search_regex_ignorecase = re.compile("|".join(_search_regex_parts), re.IGNORECASE)
local_tz_offset = get_local_tz_offset()



info = {
    "name": "lag",
    "date_order": "DMY",
    "january": [
        "fúngatɨ",
        "kʉfúngatɨ"
    ],
    "february": [
        "kʉnaanɨ",
        "naanɨ"
    ],
    "march": [
        "keenda",
        "kʉkeenda"
    ],
    "april": [
        "ikúmi",
        "kwiikumi"
    ],
    "may": [
        "inyambala",
        "kwiinyambála"
    ],
    "june": [
        "idwaata",
        "kwiidwaata"
    ],
    "july": [
        "kʉmʉʉnchɨ",
        "mʉʉnchɨ"
    ],
    "august": [
        "kʉvɨɨrɨ",
        "vɨɨrɨ"
    ],
    "september": [
        "kʉsaatʉ",
        "saatʉ"
    ],
    "october": [
        "inyi",
        "kwiinyi"
    ],
    "november": [
        "kʉsaano",
        "saano"
    ],
    "december": [
        "kʉsasatʉ",
        "sasatʉ"
    ],
    "monday": [
        "jumatátu",
        "táatu"
    ],
    "tuesday": [
        "jumaíne",
        "íne"
    ],
    "wednesday": [
        "jumatáano",
        "táano"
    ],
    "thursday": [
        "alamíisi",
        "alh"
    ],
    "friday": [
        "ijm",
        "ijumáa"
    ],
    "saturday": [
        "jumamóosi",
        "móosi"
    ],
    "sunday": [
        "jumapíiri",
        "píili"
    ],
    "am": [
        "too"
    ],
    "pm": [
        "muu"
    ],
    "year": [
        "mwaáka"
    ],
    "month": [
        "mweéri"
    ],
    "week": [
        "wíiki"
    ],
    "day": [
        "sikʉ"
    ],
    "hour": [
        "sáa"
    ],
    "minute": [
        "dakíka"
    ],
    "second": [
        "sekúunde"
    ],
    "relative-type": {
        "0 day ago": [
            "isikʉ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "niijo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "lamʉtoondo"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "eo",
    "date_order": "YMD",
    "january": [
        "jan",
        "januaro"
    ],
    "february": [
        "feb",
        "februaro"
    ],
    "march": [
        "mar",
        "marto"
    ],
    "april": [
        "apr",
        "aprilo"
    ],
    "may": [
        "maj",
        "majo"
    ],
    "june": [
        "jun",
        "junio"
    ],
    "july": [
        "jul",
        "julio"
    ],
    "august": [
        "aŭg",
        "aŭgusto"
    ],
    "september": [
        "sep",
        "septembro"
    ],
    "october": [
        "okt",
        "oktobro"
    ],
    "november": [
        "nov",
        "novembro"
    ],
    "december": [
        "dec",
        "decembro"
    ],
    "monday": [
        "lu",
        "lundo"
    ],
    "tuesday": [
        "ma",
        "mardo"
    ],
    "wednesday": [
        "me",
        "merkredo"
    ],
    "thursday": [
        "ĵa",
        "ĵaŭdo"
    ],
    "friday": [
        "ve",
        "vendredo"
    ],
    "saturday": [
        "sa",
        "sabato"
    ],
    "sunday": [
        "di",
        "dimanĉo"
    ],
    "am": [
        "atm"
    ],
    "pm": [
        "ptm"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "yav",
    "date_order": "DMY",
    "january": [
        "o1",
        "pikítíkítie, oólí ú kutúan"
    ],
    "february": [
        "o2",
        "siɛyɛ́, oóli ú kándíɛ"
    ],
    "march": [
        "o3",
        "ɔnsúmbɔl, oóli ú kátátúɛ"
    ],
    "april": [
        "mesiŋ, oóli ú kénie",
        "o4"
    ],
    "may": [
        "ensil, oóli ú kátánuɛ",
        "o5"
    ],
    "june": [
        "o6",
        "ɔsɔn"
    ],
    "july": [
        "efute",
        "o7"
    ],
    "august": [
        "o8",
        "pisuyú"
    ],
    "september": [
        "imɛŋ i puɔs",
        "o9"
    ],
    "october": [
        "imɛŋ i putúk,oóli ú kátíɛ",
        "o10"
    ],
    "november": [
        "makandikɛ",
        "o11"
    ],
    "december": [
        "o12",
        "pilɔndɔ́"
    ],
    "monday": [
        "md",
        "móndie"
    ],
    "tuesday": [
        "muányáŋmóndie",
        "mw"
    ],
    "wednesday": [
        "et",
        "metúkpíápɛ"
    ],
    "thursday": [
        "kl",
        "kúpélimetúkpiapɛ"
    ],
    "friday": [
        "feléte",
        "fl"
    ],
    "saturday": [
        "ss",
        "séselé"
    ],
    "sunday": [
        "sd",
        "sɔ́ndiɛ"
    ],
    "am": [
        "kiɛmɛ́ɛm"
    ],
    "pm": [
        "kisɛ́ndɛ"
    ],
    "year": [
        "yɔɔŋ"
    ],
    "month": [
        "oóli"
    ],
    "week": [
        "sɔ́ndiɛ"
    ],
    "day": [
        "puɔ́sɛ́"
    ],
    "hour": [
        "kisikɛl,"
    ],
    "minute": [
        "minít"
    ],
    "second": [
        "síkɛn"
    ],
    "relative-type": {
        "0 day ago": [
            "ínaan"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "púyoó"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "nakinyám"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import sys

from tzlocal import get_localzone

from .conf import apply_settings
from .timezone_parser import pop_tz_offset_from_string
from .utils import apply_timezone, localize_timezone, strip_braces


class DateParser:
    @apply_settings
    def parse(self, date_string, parse_method, settings=None):
        date_string = str(date_string)

        if not date_string.strip():
            raise ValueError("Empty string")

        date_string = strip_braces(date_string)
        date_string, ptz = pop_tz_offset_from_string(date_string)

        date_obj, period = parse_method(date_string, settings=settings, tz=ptz)

        _settings_tz = settings.TIMEZONE.lower()

        if ptz:
            if hasattr(ptz, "localize"):
                date_obj = ptz.localize(date_obj)
            else:
                date_obj = date_obj.replace(tzinfo=ptz)
            if "local" not in _settings_tz:
                date_obj = apply_timezone(date_obj, settings.TIMEZONE)
        else:
            if "local" in _settings_tz:
                stz = get_localzone()
                if hasattr(stz, "localize") and sys.version_info < (3, 6):
                    date_obj = stz.localize(date_obj)
                else:
                    date_obj = date_obj.replace(tzinfo=stz)
            else:
                date_obj = localize_timezone(date_obj, settings.TIMEZONE)

        if settings.TO_TIMEZONE:
            date_obj = apply_timezone(date_obj, settings.TO_TIMEZONE)

        if not settings.RETURN_AS_TIMEZONE_AWARE or (
            settings.RETURN_AS_TIMEZONE_AWARE
            and "default" == settings.RETURN_AS_TIMEZONE_AWARE
            and not ptz
        ):
            date_obj = date_obj.replace(tzinfo=None)

        return date_obj, period


date_parser = DateParser()



import regex as re

from dateparser.utils import get_logger


class LanguageValidator:
    logger = None

    VALID_KEYS = [
        "name",
        "skip",
        "pertain",
        "simplifications",
        "no_word_spacing",
        "ago",
        "in",
        "monday",
        "tuesday",
        "wednesday",
        "thursday",
        "friday",
        "saturday",
        "sunday",
        "january",
        "february",
        "march",
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
        "year",
        "month",
        "week",
        "day",
        "hour",
        "minute",
        "second",
        "sentence_splitter_group",
    ]

    @classmethod
    def get_logger(cls):
        if cls.logger is None:
            cls.logger = get_logger()
        return cls.logger

    @classmethod
    def validate_info(cls, language_id, info):
        result = True

        result &= cls._validate_type(language_id, info)
        if not result:
            return False

        result &= cls._validate_name(language_id, info)
        result &= cls._validate_word_spacing(language_id, info)
        result &= cls._validate_skip_list(language_id, info)
        result &= cls._validate_pertain_list(language_id, info)
        result &= cls._validate_weekdays(language_id, info)
        result &= cls._validate_months(language_id, info)
        result &= cls._validate_units(language_id, info)
        result &= cls._validate_other_words(language_id, info)
        result &= cls._validate_simplifications(language_id, info)
        result &= cls._validate_extra_keys(language_id, info)
        return result

    @classmethod
    def _validate_type(cls, language_id, info):
        result = True

        if not isinstance(info, dict):
            cls.get_logger().error(
                "Language '%(id)s' info expected to be dict, but have got %(type)s",
                {"id": language_id, "type": type(info).__name__},
            )
            result = False

        return result

    @classmethod
    def _validate_name(cls, language_id, info):
        result = True

        if "name" not in info or not isinstance(info["name"], str) or not info["name"]:
            cls.get_logger().error(
                "Language '%(id)s' does not have a name", {"id": language_id}
            )
            result = False

        return result

    @classmethod
    def _validate_word_spacing(cls, language_id, info):
        if "no_word_spacing" not in info:
            return True  # Optional key

        result = True

        value = info["no_word_spacing"]
        if value not in [True, False]:
            cls.get_logger().error(
                "Invalid 'no_word_spacing' value %(value)r for '%(id)s' language: "
                "expected boolean",
                {"value": value, "id": language_id},
            )
            result = False

        return result

    @classmethod
    def _validate_sentence_splitter_group(cls, language_id, info):
        if "sentence_splitter_group" not in info:
            return True  # Optional key

        result = True

        group = info["sentence_splitter_group"]
        if isinstance(group, int) or not group:
            if group < 1 or group > 6:
                cls.get_logger().error(
                    "Invalid 'sentence_splitter_group' number %(number)r for '%(id)s' language: "
                    "expected number from 1 to 6",
                    {"number": group, "id": language_id},
                )
                result = False
        else:
            cls.get_logger().error(
                "Invalid 'sentence_splitter_group' for '%(id)s' language: "
                "expected int type but have got %(type)s",
                {"id": language_id, "type": type(group).__name__},
            )
            result = False

        return result

    @classmethod
    def _validate_skip_list(cls, language_id, info):
        if "skip" not in info:
            return True  # Optional key

        result = True

        skip_tokens_list = info["skip"]
        if isinstance(skip_tokens_list, list):
            for token in skip_tokens_list:
                if not isinstance(token, str) or not token:
                    cls.get_logger().error(
                        "Invalid 'skip' token %(token)r for '%(id)s' language: "
                        "expected not empty string",
                        {"token": token, "id": language_id},
                    )
                    result = False
        else:
            cls.get_logger().error(
                "Invalid 'skip' list for '%(id)s' language: "
                "expected list type but have got %(type)s",
                {"id": language_id, "type": type(skip_tokens_list).__name__},
            )
            result = False

        return result

    @classmethod
    def _validate_pertain_list(cls, language_id, info):
        if "pertain" not in info:
            return True  # Optional key

        result = True

        pertain_tokens_list = info["skip"]
        if isinstance(pertain_tokens_list, list):
            for token in pertain_tokens_list:
                if not isinstance(token, str) or not token:
                    cls.get_logger().error(
                        "Invalid 'pertain' token %(token)r for '%(id)s' language: "
                        "expected not empty string",
                        {"token": token, "id": language_id},
                    )
                    result = False
        else:
            cls.get_logger().error(
                "Invalid 'pertain' list for '%(id)s' language: "
                "expected list type but have got %(type)s",
                {"id": language_id, "type": type(pertain_tokens_list).__name__},
            )
            result = False

        return result

    @classmethod
    def _validate_weekdays(cls, language_id, info):
        result = True

        for weekday in (
            "monday",
            "tuesday",
            "wednesday",
            "thursday",
            "friday",
            "saturday",
            "sunday",
        ):
            if weekday not in info or not info[weekday]:
                cls.get_logger().error(
                    "No translations for '%(weekday)s' provided for '%(id)s' language",
                    {"weekday": weekday, "id": language_id},
                )
                result = False
                continue

            translations_list = info[weekday]
            if isinstance(translations_list, list):
                for token in translations_list:
                    if not isinstance(token, str) or not token:
                        cls.get_logger().error(
                            "Invalid '%(weekday)s' translation %(token)r for '%(id)s' language: "
                            "expected not empty string",
                            {"weekday": weekday, "token": token, "id": language_id},
                        )
                        result = False
            else:
                cls.get_logger().error(
                    "Invalid '%(weekday)s' translations list for '%(id)s' language: "
                    "expected list type but have got %(type)s",
                    {
                        "weekday": weekday,
                        "id": language_id,
                        "type": type(translations_list).__name__,
                    },
                )
                result = False

        return result

    @classmethod
    def _validate_months(cls, language_id, info):
        result = True

        for month in (
            "january",
            "february",
            "march",
            "april",
            "may",
            "june",
            "july",
            "august",
            "september",
            "october",
            "november",
            "december",
        ):
            if month not in info or not info[month]:
                cls.get_logger().error(
                    "No translations for '%(month)s' provided for '%(id)s' language",
                    {"month": month, "id": language_id},
                )
                result = False
                continue

            translations_list = info[month]
            if isinstance(translations_list, list):
                for token in translations_list:
                    if not isinstance(token, str) or not token:
                        cls.get_logger().error(
                            "Invalid '%(month)s' translation %(token)r for '%(id)s' language: "
                            "expected not empty string",
                            {"month": month, "token": token, "id": language_id},
                        )
                        result = False
            else:
                cls.get_logger().error(
                    "Invalid '%(month)s' translations list for '%(id)s' language: "
                    "expected list type but have got %(type)s",
                    {
                        "month": month,
                        "id": language_id,
                        "type": type(translations_list).__name__,
                    },
                )
                result = False

        return result

    @classmethod
    def _validate_units(cls, language_id, info):
        result = True

        for unit in "year", "month", "week", "day", "hour", "minute", "second":
            if unit not in info or not info[unit]:
                cls.get_logger().error(
                    "No translations for '%(unit)s' provided for '%(id)s' language",
                    {"unit": unit, "id": language_id},
                )
                result = False
                continue

            translations_list = info[unit]
            if isinstance(translations_list, list):
                for token in translations_list:
                    if not isinstance(token, str) or not token:
                        cls.get_logger().error(
                            "Invalid '%(unit)s' translation %(token)r for '%(id)s' language: "
                            "expected not empty string",
                            {"unit": unit, "token": token, "id": language_id},
                        )
                        result = False
            else:
                cls.get_logger().error(
                    "Invalid '%(unit)s' translations list for '%(id)s' language: "
                    "expected list type but have got %(type)s",
                    {
                        "unit": unit,
                        "id": language_id,
                        "type": type(translations_list).__name__,
                    },
                )
                result = False

        return result

    @classmethod
    def _validate_other_words(cls, language_id, info):
        result = True

        for word in ("ago",):
            if word not in info or not info[word]:
                cls.get_logger().error(
                    "No translations for '%(word)s' provided for '%(id)s' language",
                    {"word": word, "id": language_id},
                )
                result = False
                continue

            translations_list = info[word]
            if isinstance(translations_list, list):
                for token in translations_list:
                    if not isinstance(token, str) or not token:
                        cls.get_logger().error(
                            "Invalid '%(word)s' translation %(token)r for '%(id)s' language: "
                            "expected not empty string",
                            {"word": word, "token": token, "id": language_id},
                        )
                        result = False
            else:
                cls.get_logger().error(
                    "Invalid '%(word)s' translations list for '%(id)s' language: "
                    "expected list type but have got %(type)s",
                    {
                        "word": word,
                        "id": language_id,
                        "type": type(translations_list).__name__,
                    },
                )
                result = False

        return result

    @classmethod
    def _validate_simplifications(cls, language_id, info):
        if "simplifications" not in info:
            return True  # Optional key

        result = True

        simplifications_list = info["simplifications"]
        if isinstance(simplifications_list, list):
            for simplification in simplifications_list:
                if not isinstance(simplification, dict) or len(simplification) != 1:
                    cls.get_logger().error(
                        "Invalid simplification %(simplification)r for '%(id)s' language: "
                        "eash simplification suppose to be one-to-one mapping",
                        {"simplification": simplification, "id": language_id},
                    )
                    result = False
                    continue

                key, value = list(simplification.items())[0]
                if not isinstance(key, str) or not isinstance(value, (str, int)):
                    cls.get_logger().error(
                        "Invalid simplification %(simplification)r for '%(id)s' language: "
                        "each simplification suppose to be string-to-string-or-int mapping",
                        {"simplification": simplification, "id": language_id},
                    )
                    result = False
                    continue

                compiled_key = re.compile(key)
                value = str(value)
                replacements = re.findall(r"\\(\d+)", value)
                replacements.extend(re.findall(r"\\g<(.+?)>", value))

                groups = []
                for group in replacements:
                    if group.isdigit():
                        groups.append(int(group))
                    elif group in compiled_key.groupindex:
                        groups.append(compiled_key.groupindex[group])
                    else:
                        cls.get_logger().error(
                            "Invalid simplification %(simplification)r for '%(id)s' language: "
                            "unknown group %(group)s",
                            {
                                "simplification": simplification,
                                "id": language_id,
                                "group": group,
                            },
                        )
                        result = False

                used_groups = set(map(int, groups))
                expected_groups = set(range(0, compiled_key.groups + 1))
                extra_groups = used_groups - expected_groups
                not_used_groups = expected_groups - used_groups
                not_used_groups -= {0}  # Entire substring is not required to be used

                if extra_groups:
                    cls.get_logger().error(
                        "Invalid simplification %(simplification)r for '%(id)s' language: "
                        "unknown groups %(groups)s",
                        {
                            "simplification": simplification,
                            "id": language_id,
                            "groups": ", ".join(map(str, sorted(extra_groups))),
                        },
                    )
                    result = False

                if not_used_groups:
                    cls.get_logger().error(
                        "Invalid simplification %(simplification)r for '%(id)s' language: "
                        "groups %(groups)s were not used",
                        {
                            "simplification": simplification,
                            "id": language_id,
                            "groups": ", ".join(map(str, sorted(not_used_groups))),
                        },
                    )
                    result = False
        else:
            cls.get_logger().error(
                "Invalid 'simplifications' list for '%(id)s' language: "
                "expected list type but have got %(type)s",
                {"id": language_id, "type": type(simplifications_list).__name__},
            )
            result = False

        return result

    @classmethod
    def _validate_extra_keys(cls, language_id, info):
        result = True

        extra_keys = set(info.keys()) - set(cls.VALID_KEYS)
        if extra_keys:
            cls.get_logger().error(
                "Extra keys found for '%(id)s' language: %(keys)s",
                {"id": language_id, "keys": ", ".join(map(repr, extra_keys))},
            )
            result = False

        return result



from hijridate import Gregorian, Hijri

from dateparser.calendars import non_gregorian_parser


class hijri:
    @classmethod
    def to_gregorian(cls, year=None, month=None, day=None):
        g = Hijri(year=year, month=month, day=day, validate=False).to_gregorian()
        return g.datetuple()

    @classmethod
    def from_gregorian(cls, year=None, month=None, day=None):
        h = Gregorian(year, month, day).to_hijri()
        return h.datetuple()

    @classmethod
    def month_length(cls, year, month):
        h = Hijri(year=year, month=month, day=1)
        return h.month_length()


class HijriDate:
    def __init__(self, year, month, day):
        self.year = year
        self.month = month
        self.day = day

    def weekday(self):
        for week in hijri.monthcalendar(self.year, self.month):
            for idx, day in enumerate(week):
                if day == self.day:
                    return idx


class hijri_parser(non_gregorian_parser):
    calendar_converter = hijri
    default_year = 1389
    default_month = 1
    default_day = 1
    non_gregorian_date_cls = HijriDate

    _time_conventions = {
        "am": ["صباحاً"],
        "pm": ["مساءً"],
    }

    @classmethod
    def _replace_time_conventions(cls, source):
        result = source
        for latin, arabics in cls._time_conventions.items():
            for arabic in arabics:
                result = result.replace(arabic, latin)
        return result

    def handle_two_digit_year(self, year):
        if year >= 90:
            return year + 1300
        else:
            return year + 1400



info = {
    "name": "sbp",
    "date_order": "DMY",
    "january": [
        "mup",
        "mupalangulwa"
    ],
    "february": [
        "mwi",
        "mwitope"
    ],
    "march": [
        "msh",
        "mushende"
    ],
    "april": [
        "mun",
        "munyi"
    ],
    "may": [
        "mag",
        "mushende magali"
    ],
    "june": [
        "muj",
        "mujimbi"
    ],
    "july": [
        "msp",
        "mushipepo"
    ],
    "august": [
        "mpg",
        "mupuguto"
    ],
    "september": [
        "munyense",
        "mye"
    ],
    "october": [
        "mok",
        "mokhu"
    ],
    "november": [
        "mus",
        "musongandembwe"
    ],
    "december": [
        "muh",
        "muhaano"
    ],
    "monday": [
        "jtt",
        "jumatatu"
    ],
    "tuesday": [
        "jnn",
        "jumanne"
    ],
    "wednesday": [
        "jtn",
        "jumatano"
    ],
    "thursday": [
        "alahamisi",
        "alh"
    ],
    "friday": [
        "iju",
        "ijumaa"
    ],
    "saturday": [
        "jmo",
        "jumamosi"
    ],
    "sunday": [
        "mul",
        "mulungu"
    ],
    "am": [
        "lwamilawu"
    ],
    "pm": [
        "pashamihe"
    ],
    "year": [
        "mwakha"
    ],
    "month": [
        "mwesi"
    ],
    "week": [
        "ilijuma"
    ],
    "day": [
        "lusiku"
    ],
    "hour": [
        "ilisala"
    ],
    "minute": [
        "idakika"
    ],
    "second": [
        "isekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "ineng'uni"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "imehe"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "pamulaawu"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ff",
    "date_order": "DMY",
    "january": [
        "sii",
        "siilo"
    ],
    "february": [
        "col",
        "colte"
    ],
    "march": [
        "mbo",
        "mbooy"
    ],
    "april": [
        "see",
        "seeɗto"
    ],
    "may": [
        "duu",
        "duujal"
    ],
    "june": [
        "kor",
        "korse"
    ],
    "july": [
        "mor",
        "morso"
    ],
    "august": [
        "juk",
        "juko"
    ],
    "september": [
        "siilto",
        "slt"
    ],
    "october": [
        "yar",
        "yarkomaa"
    ],
    "november": [
        "jol",
        "jolal"
    ],
    "december": [
        "bow",
        "bowte"
    ],
    "monday": [
        "aaɓ",
        "aaɓnde"
    ],
    "tuesday": [
        "maw",
        "mawbaare"
    ],
    "wednesday": [
        "nje",
        "njeslaare"
    ],
    "thursday": [
        "naa",
        "naasaande"
    ],
    "friday": [
        "mawnde",
        "mwd"
    ],
    "saturday": [
        "hbi",
        "hoore-biir"
    ],
    "sunday": [
        "dew",
        "dewo"
    ],
    "am": [
        "subaka"
    ],
    "pm": [
        "kikiiɗe"
    ],
    "year": [
        "hitaande"
    ],
    "month": [
        "lewru"
    ],
    "week": [
        "yontere"
    ],
    "day": [
        "ñalnde"
    ],
    "hour": [
        "waktu"
    ],
    "minute": [
        "hoƴom"
    ],
    "second": [
        "majaango"
    ],
    "relative-type": {
        "0 day ago": [
            "hannde"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "haŋki"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "jaŋngo"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "ff-CM": {
            "name": "ff-CM"
        },
        "ff-GN": {
            "name": "ff-GN"
        },
        "ff-MR": {
            "name": "ff-MR"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "fy",
    "date_order": "DMY",
    "january": [
        "jan",
        "jannewaris"
    ],
    "february": [
        "feb",
        "febrewaris"
    ],
    "march": [
        "maart",
        "mrt"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "maaie",
        "mai"
    ],
    "june": [
        "jun",
        "juny"
    ],
    "july": [
        "jul",
        "july"
    ],
    "august": [
        "aug",
        "augustus"
    ],
    "september": [
        "sep",
        "septimber"
    ],
    "october": [
        "okt",
        "oktober"
    ],
    "november": [
        "nov",
        "novimber"
    ],
    "december": [
        "des",
        "desimber"
    ],
    "monday": [
        "mo",
        "moandei"
    ],
    "tuesday": [
        "ti",
        "tiisdei"
    ],
    "wednesday": [
        "wo",
        "woansdei"
    ],
    "thursday": [
        "to",
        "tongersdei"
    ],
    "friday": [
        "fr",
        "freed"
    ],
    "saturday": [
        "sneon",
        "so"
    ],
    "sunday": [
        "si",
        "snein"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "jier"
    ],
    "month": [
        "moanne"
    ],
    "week": [
        "wike"
    ],
    "day": [
        "dei"
    ],
    "hour": [
        "oere"
    ],
    "minute": [
        "minút"
    ],
    "second": [
        "sekonde"
    ],
    "relative-type": {
        "0 day ago": [
            "vandaag"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "dizze moanne"
        ],
        "0 second ago": [
            "nu"
        ],
        "0 week ago": [
            "dizze wike"
        ],
        "0 year ago": [
            "dit jier"
        ],
        "1 day ago": [
            "gisteren"
        ],
        "1 month ago": [
            "foarige moanne"
        ],
        "1 week ago": [
            "foarige wike"
        ],
        "1 year ago": [
            "foarich jier"
        ],
        "in 1 day": [
            "morgen"
        ],
        "in 1 month": [
            "folgjende moanne"
        ],
        "in 1 week": [
            "folgjende wike"
        ],
        "in 1 year": [
            "folgjend jier"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) dei lyn",
            "(\\d+[.,]?\\d*) deien lyn"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) oere lyn"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) minuten lyn",
            "(\\d+[.,]?\\d*) minút lyn"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) moanne lyn",
            "(\\d+[.,]?\\d*) moannen lyn"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) sekonde lyn",
            "(\\d+[.,]?\\d*) sekonden lyn"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) wike lyn",
            "(\\d+[.,]?\\d*) wiken lyn"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) jier lyn"
        ],
        "in \\1 day": [
            "oer (\\d+[.,]?\\d*) dei",
            "oer (\\d+[.,]?\\d*) deien"
        ],
        "in \\1 hour": [
            "oer (\\d+[.,]?\\d*) oere"
        ],
        "in \\1 minute": [
            "oer (\\d+[.,]?\\d*) minuten",
            "oer (\\d+[.,]?\\d*) minút"
        ],
        "in \\1 month": [
            "oer (\\d+[.,]?\\d*) moanne",
            "oer (\\d+[.,]?\\d*) moannen"
        ],
        "in \\1 second": [
            "oer (\\d+[.,]?\\d*) sekonde",
            "oer (\\d+[.,]?\\d*) sekonden"
        ],
        "in \\1 week": [
            "oer (\\d+[.,]?\\d*) wike",
            "oer (\\d+[.,]?\\d*) wiken"
        ],
        "in \\1 year": [
            "oer (\\d+[.,]?\\d*) jier"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "shi-Tfng",
    "date_order": "DMY",
    "january": [
        "ⵉⵏⵏ",
        "ⵉⵏⵏⴰⵢⵔ"
    ],
    "february": [
        "ⴱⵕⴰ",
        "ⴱⵕⴰⵢⵕ"
    ],
    "march": [
        "ⵎⴰⵕ",
        "ⵎⴰⵕⵚ"
    ],
    "april": [
        "ⵉⴱⵔ",
        "ⵉⴱⵔⵉⵔ"
    ],
    "may": [
        "ⵎⴰⵢ",
        "ⵎⴰⵢⵢⵓ"
    ],
    "june": [
        "ⵢⵓⵏ",
        "ⵢⵓⵏⵢⵓ"
    ],
    "july": [
        "ⵢⵓⵍ",
        "ⵢⵓⵍⵢⵓⵣ"
    ],
    "august": [
        "ⵖⵓⵛ",
        "ⵖⵓⵛⵜ"
    ],
    "september": [
        "ⵛⵓⵜ",
        "ⵛⵓⵜⴰⵏⴱⵉⵔ"
    ],
    "october": [
        "ⴽⵜⵓ",
        "ⴽⵜⵓⴱⵔ"
    ],
    "november": [
        "ⵏⵓⵡ",
        "ⵏⵓⵡⴰⵏⴱⵉⵔ"
    ],
    "december": [
        "ⴷⵓⵊ",
        "ⴷⵓⵊⴰⵏⴱⵉⵔ"
    ],
    "monday": [
        "ⴰⵢⵏ",
        "ⴰⵢⵏⴰⵙ"
    ],
    "tuesday": [
        "ⴰⵙⵉ",
        "ⴰⵙⵉⵏⴰⵙ"
    ],
    "wednesday": [
        "ⴰⴽⵕ",
        "ⴰⴽⵕⴰⵙ"
    ],
    "thursday": [
        "ⴰⴽⵡ",
        "ⴰⴽⵡⴰⵙ"
    ],
    "friday": [
        "ⴰⵙⵉⵎ",
        "ⵙⵉⵎⵡⴰⵙ"
    ],
    "saturday": [
        "ⴰⵙⵉⴹ",
        "ⴰⵙⵉⴹⵢⴰⵙ"
    ],
    "sunday": [
        "ⴰⵙⴰ",
        "ⴰⵙⴰⵎⴰⵙ"
    ],
    "am": [
        "ⵜⵉⴼⴰⵡⵜ"
    ],
    "pm": [
        "ⵜⴰⴷⴳⴳⵯⴰⵜ"
    ],
    "year": [
        "ⴰⵙⴳⴳⵯⴰⵙ"
    ],
    "month": [
        "ⴰⵢⵢⵓⵔ"
    ],
    "week": [
        "ⵉⵎⴰⵍⴰⵙⵙ"
    ],
    "day": [
        "ⴰⵙⵙ"
    ],
    "hour": [
        "ⵜⴰⵙⵔⴰⴳⵜ"
    ],
    "minute": [
        "ⵜⵓⵙⴷⵉⴷⵜ"
    ],
    "second": [
        "ⵜⴰⵙⵉⵏⵜ"
    ],
    "relative-type": {
        "0 day ago": [
            "ⴰⵙⵙⴰ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ⵉⴹⵍⵍⵉ"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ⴰⵙⴽⴽⴰ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import logging
import os
import urllib.request
from pathlib import Path

from .exceptions import FastTextModelNotFoundException
from .utils import create_data_model_home, dateparser_model_home


def fasttext_downloader(model_name):
    create_data_model_home()
    models = {
        "small": "https://dl.fbaipublicfiles.com/fasttext/supervised-models/lid.176.ftz",
        "large": "https://dl.fbaipublicfiles.com/fasttext/supervised-models/lid.176.bin",
    }
    if model_name not in models:
        message = 'dateparser-download: Couldn\'t find a model called "{}". Supported models are: {}'.format(
            model_name, ", ".join(models.keys())
        )
        raise FastTextModelNotFoundException(message)

    models_directory_path = os.path.join(dateparser_model_home, (model_name + ".bin"))

    if not Path(models_directory_path).is_file():
        model_url = models[model_name]
        logging.info(
            'dateparser-download: Downloading model "{}" from "{}"...'.format(
                model_name, model_url
            )
        )
        try:
            urllib.request.urlretrieve(model_url, models_directory_path)
        except urllib.error.HTTPError as e:
            raise Exception(
                "dateparser-download: Fasttext model cannot be downloaded due to HTTP error"
            ) from e
    else:
        logging.info(
            'dateparser-download: The model "{}" is already downloaded'.format(
                model_name
            )
        )



info = {
    "name": "is",
    "date_order": "DMY",
    "january": [
        "jan",
        "janúar"
    ],
    "february": [
        "feb",
        "febrúar"
    ],
    "march": [
        "mar",
        "mars"
    ],
    "april": [
        "apr",
        "apríl"
    ],
    "may": [
        "maí"
    ],
    "june": [
        "jún",
        "júní"
    ],
    "july": [
        "júl",
        "júlí"
    ],
    "august": [
        "ágú",
        "ágúst"
    ],
    "september": [
        "sep",
        "september"
    ],
    "october": [
        "okt",
        "október"
    ],
    "november": [
        "nóv",
        "nóvember"
    ],
    "december": [
        "des",
        "desember"
    ],
    "monday": [
        "mán",
        "mánudagur"
    ],
    "tuesday": [
        "þri",
        "þriðjudagur"
    ],
    "wednesday": [
        "mið",
        "miðvikudagur"
    ],
    "thursday": [
        "fim",
        "fimmtudagur"
    ],
    "friday": [
        "fös",
        "föstudagur"
    ],
    "saturday": [
        "lau",
        "laugardagur"
    ],
    "sunday": [
        "sun",
        "sunnudagur"
    ],
    "am": [
        "fh"
    ],
    "pm": [
        "eh"
    ],
    "year": [
        "ár"
    ],
    "month": [
        "mán",
        "mánuður"
    ],
    "week": [
        "v",
        "vika"
    ],
    "day": [
        "d",
        "dagur"
    ],
    "hour": [
        "klst",
        "klukkustund"
    ],
    "minute": [
        "mín",
        "mínúta"
    ],
    "second": [
        "sek",
        "sekúnda"
    ],
    "relative-type": {
        "0 day ago": [
            "í dag"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "í þessum mán",
            "í þessum mánuði"
        ],
        "0 second ago": [
            "núna"
        ],
        "0 week ago": [
            "í þessari viku"
        ],
        "0 year ago": [
            "á þessu ári"
        ],
        "1 day ago": [
            "í gær"
        ],
        "1 month ago": [
            "í síðasta mán",
            "í síðasta mánuði"
        ],
        "1 week ago": [
            "í síðustu viku"
        ],
        "1 year ago": [
            "á síðasta ári"
        ],
        "in 1 day": [
            "á morgun"
        ],
        "in 1 month": [
            "í næsta mán",
            "í næsta mánuði"
        ],
        "in 1 week": [
            "í næstu viku"
        ],
        "in 1 year": [
            "á næsta ári"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "fyrir (\\d+[.,]?\\d*) degi",
            "fyrir (\\d+[.,]?\\d*) dögum"
        ],
        "\\1 hour ago": [
            "fyrir (\\d+[.,]?\\d*) klst",
            "fyrir (\\d+[.,]?\\d*) klukkustund",
            "fyrir (\\d+[.,]?\\d*) klukkustundum"
        ],
        "\\1 minute ago": [
            "fyrir (\\d+[.,]?\\d*) mín",
            "fyrir (\\d+[.,]?\\d*) mínútu",
            "fyrir (\\d+[.,]?\\d*) mínútum"
        ],
        "\\1 month ago": [
            "fyrir (\\d+[.,]?\\d*) mán",
            "fyrir (\\d+[.,]?\\d*) mánuði",
            "fyrir (\\d+[.,]?\\d*) mánuðum"
        ],
        "\\1 second ago": [
            "fyrir (\\d+[.,]?\\d*) sek",
            "fyrir (\\d+[.,]?\\d*) sekúndu",
            "fyrir (\\d+[.,]?\\d*) sekúndum"
        ],
        "\\1 week ago": [
            "fyrir (\\d+[.,]?\\d*) viku",
            "fyrir (\\d+[.,]?\\d*) vikum"
        ],
        "\\1 year ago": [
            "fyrir (\\d+[.,]?\\d*) ári",
            "fyrir (\\d+[.,]?\\d*) árum"
        ],
        "in \\1 day": [
            "eftir (\\d+[.,]?\\d*) dag",
            "eftir (\\d+[.,]?\\d*) daga"
        ],
        "in \\1 hour": [
            "eftir (\\d+[.,]?\\d*) klst",
            "eftir (\\d+[.,]?\\d*) klukkustund",
            "eftir (\\d+[.,]?\\d*) klukkustundir"
        ],
        "in \\1 minute": [
            "eftir (\\d+[.,]?\\d*) mín",
            "eftir (\\d+[.,]?\\d*) mínútu",
            "eftir (\\d+[.,]?\\d*) mínútur"
        ],
        "in \\1 month": [
            "eftir (\\d+[.,]?\\d*) mán",
            "eftir (\\d+[.,]?\\d*) mánuð",
            "eftir (\\d+[.,]?\\d*) mánuði"
        ],
        "in \\1 second": [
            "eftir (\\d+[.,]?\\d*) sek",
            "eftir (\\d+[.,]?\\d*) sekúndu",
            "eftir (\\d+[.,]?\\d*) sekúndur"
        ],
        "in \\1 week": [
            "eftir (\\d+[.,]?\\d*) viku",
            "eftir (\\d+[.,]?\\d*) vikur"
        ],
        "in \\1 year": [
            "eftir (\\d+[.,]?\\d*) ár"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from collections import OrderedDict
from itertools import chain

import regex as re
from dateutil import parser

from dateparser.timezone_parser import pop_tz_offset_from_string, word_is_tz
from dateparser.utils import combine_dicts, normalize_unicode

from .dictionary import ALWAYS_KEEP_TOKENS, Dictionary, NormalizedDictionary

NUMERAL_PATTERN = re.compile(r"(\d+)", re.U)


def _parse_bool(value):
    if isinstance(value, bool):
        return value
    if isinstance(value, str):
        return value.strip().lower() in ("true", "1", "yes")
    return False


class Locale:
    """
    Class that deals with applicability and translation from a locale.

    :param shortname:
        A locale code, e.g. 'fr-PF', 'qu-EC', 'af-NA'.
    :type shortname: str

    :param language_info:
        Language info (translation data) of the language the locale belongs to.
    :type language_info: dict

    :return: A Locale instance
    """

    _dictionary = None
    _normalized_dictionary = None
    _simplifications = None
    _normalized_simplifications = None
    _splitters = None
    _wordchars = None
    _relative_translations = None
    _normalized_relative_translations = None
    _abbreviations = None
    _split_dictionary = None
    _wordchars_for_detection = None

    def __init__(self, shortname, language_info):
        self.shortname = shortname
        locale_specific_info = language_info.get("locale_specific", {}).get(
            shortname, {}
        )
        self.info = combine_dicts(language_info, locale_specific_info)
        self.info.pop("locale_specific", None)

    def is_applicable(self, date_string, strip_timezone=False, settings=None):
        """
        Check if the locale is applicable to translate date string.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str

        :param strip_timezone:
            If True, timezone is stripped from date string.
        :type strip_timezone: bool

        :return: boolean value representing if the locale is applicable for the date string or not.
        """
        if strip_timezone:
            date_string, _ = pop_tz_offset_from_string(date_string, as_offset=False)

        date_string = self._translate_numerals(date_string)
        if settings.NORMALIZE:
            date_string = normalize_unicode(date_string)
        date_string = self._simplify(date_string, settings=settings)
        dictionary = self._get_dictionary(settings)
        date_tokens = dictionary.split(date_string)
        return dictionary.are_tokens_valid(date_tokens)

    def count_applicability(self, text, strip_timezone=False, settings=None):
        if strip_timezone:
            text, _ = pop_tz_offset_from_string(text, as_offset=False)

        text = self._simplify(text, settings=settings)
        sentences = self._sentence_split(text, settings=settings)
        tokens = []
        for sent in sentences:
            tokens.extend(self._split(sent, keep_formatting=False, settings=settings))
        return self._count_words_present_in_the_dictionary(tokens, settings)

    def _count_words_present_in_the_dictionary(self, words, settings=None):
        dictionary = self.clean_dictionary(
            self._get_split_dictionary(settings=settings)
        )
        dict_cnt = 0
        skip_cnt = 0
        for word in set(words):
            if word in dictionary:
                if dictionary[word]:
                    dict_cnt += 1
                else:
                    skip_cnt += 1
            elif word.isdigit():
                skip_cnt += 1
        return [dict_cnt, skip_cnt]

    @staticmethod
    def clean_dictionary(dictionary, threshold=2):
        del_keys = []
        for key in dictionary:
            if len(key) < threshold:
                del_keys.append(key)
        for del_key in del_keys:
            del dictionary[del_key]
        return dictionary

    def translate(self, date_string, keep_formatting=False, settings=None):
        """
        Translate the date string to its English equivalent.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str

        :param keep_formatting:
            If True, retain formatting of the date string after translation.
        :type keep_formatting: bool

        :return: translated date string.
        """
        date_string = self._translate_numerals(date_string)
        if settings.NORMALIZE:
            date_string = normalize_unicode(date_string)
        date_string = self._simplify(date_string, settings=settings)
        dictionary = self._get_dictionary(settings)
        date_string_tokens = dictionary.split(date_string, keep_formatting)

        relative_translations = self._get_relative_translations(settings=settings)

        for i, word in enumerate(date_string_tokens):
            word = word.lower()
            for pattern, replacement in relative_translations.items():
                if pattern.match(word):
                    date_string_tokens[i] = pattern.sub(replacement, word)
                    break
            else:
                if word in dictionary:
                    fallback = word if keep_formatting and not word.isalpha() else ""
                    date_string_tokens[i] = dictionary[word] or fallback
        if "in" in date_string_tokens:
            date_string_tokens = self._clear_future_words(date_string_tokens)
        return self._join(
            list(filter(bool, date_string_tokens)),
            separator="" if keep_formatting else " ",
            settings=settings,
        )

    def _translate_numerals(self, date_string):
        date_string_tokens = NUMERAL_PATTERN.split(date_string)
        for i, token in enumerate(date_string_tokens):
            if token.isdecimal():
                date_string_tokens[i] = str(int(token)).zfill(len(token))
        return "".join(date_string_tokens)

    def _get_relative_translations(self, settings=None):
        if settings.NORMALIZE:
            if self._normalized_relative_translations is None:
                self._normalized_relative_translations = (
                    self._generate_relative_translations(normalize=True)
                )
            return self._normalized_relative_translations
        else:
            if self._relative_translations is None:
                self._relative_translations = self._generate_relative_translations(
                    normalize=False
                )
            return self._relative_translations

    def _generate_relative_translations(self, normalize=False):
        relative_translations = self.info.get("relative-type-regex", {})
        relative_dictionary = OrderedDict()
        for key, value in relative_translations.items():
            if normalize:
                value = list(map(normalize_unicode, value))
            pattern = "|".join(sorted(value, key=len, reverse=True))
            pattern = pattern.replace(r"(\d+", r"(?P<n>\d+")
            pattern = re.compile(
                r"^(?:{})$".format(pattern), re.UNICODE | re.IGNORECASE
            )
            relative_dictionary[pattern] = key
        return relative_dictionary

    def translate_search(self, search_string, settings=None):
        dashes = ["-", "——", "—", "～"]
        word_joint_unsupported_languages = ["zh", "ja"]
        sentences = self._sentence_split(search_string, settings=settings)
        dictionary = self._get_dictionary(settings=settings)
        translated = []
        original = []
        for sentence in sentences:
            original_tokens, simplified_tokens = self._simplify_split_align(
                sentence, settings=settings
            )
            translated_chunk = []
            original_chunk = []
            last_token_index = len(simplified_tokens) - 1
            skip_next_token = False
            for i, word in enumerate(simplified_tokens):
                next_word = simplified_tokens[i + 1] if i < last_token_index else ""
                current_and_next_joined = self._join_chunk(
                    [word, next_word], settings=settings
                )
                if skip_next_token:
                    skip_next_token = False
                    continue

                if word == "" or word == " ":
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                elif (
                    current_and_next_joined in dictionary
                    and word not in dashes
                    and self.shortname not in word_joint_unsupported_languages
                ):
                    translated_chunk.append(dictionary[current_and_next_joined])
                    original_chunk.append(
                        self._join_chunk(
                            [original_tokens[i], original_tokens[i + 1]],
                            settings=settings,
                        )
                    )
                    skip_next_token = True
                elif word in dictionary and word not in dashes:
                    translated_chunk.append(dictionary[word])
                    original_chunk.append(original_tokens[i])
                elif word.strip("()\"'{}[],.،") in dictionary and word not in dashes:
                    punct = word[len(word.strip("()\"'{}[],.،")) :]
                    if punct and dictionary[word.strip("()\"'{}[],.،")]:
                        translated_chunk.append(
                            dictionary[word.strip("()\"'{}[],.،")] + punct
                        )
                    else:
                        translated_chunk.append(dictionary[word.strip("()\"'{}[],.،")])
                    original_chunk.append(original_tokens[i])
                elif self._token_with_digits_is_ok(word):
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                # Use original token because word_is_tz is case sensitive
                elif translated_chunk and word_is_tz(original_tokens[i]):
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                else:
                    if translated_chunk:
                        translated.append(translated_chunk)
                        translated_chunk = []
                        original.append(original_chunk)
                        original_chunk = []
            if translated_chunk:
                translated.append(translated_chunk)
                original.append(original_chunk)
        for i in range(len(translated)):
            if "in" in translated[i]:
                translated[i] = self._clear_future_words(translated[i])
            translated[i] = self._join_chunk(
                list(filter(bool, translated[i])), settings=settings
            )
            original[i] = self._join_chunk(
                list(filter(bool, original[i])), settings=settings
            )
        return translated, original

    def _get_abbreviations(self, settings):
        dictionary = self._get_dictionary(settings=settings)
        abbreviations = []
        if self._abbreviations is None:
            for item in dictionary:
                if item.endswith(".") and len(item) > 1:
                    abbreviations.append(item)
            self._abbreviations = abbreviations
        return self._abbreviations

    def _sentence_split(self, string, settings):
        abbreviations = self._get_abbreviations(settings=settings)
        digit_abbreviations = ["[0-9]"]  # numeric date with full stop
        abbreviation_string = ""

        for abbreviation in abbreviations:
            abbreviation_string += (
                "(?<! " + abbreviation[:-1] + ")"
            )  # negative lookbehind
        if self.shortname in ["fi", "cs", "hu", "de", "da"]:
            for digit_abbreviation in digit_abbreviations:
                abbreviation_string += (
                    "(?<!" + digit_abbreviation + ")"
                )  # negative lookbehind

        splitters_dict = {
            1: r"[\.!?;…\r\n]+(?:\s|$)*",  # most European, Tagalog, Hebrew, Georgian,
            # Indonesian, Vietnamese
            2: r"[\.!?;…\r\n]+(\s*[¡¿]*|$)|[¡¿]+",  # Spanish
            3: r"[|!?;\r\n]+(?:\s|$)+",  # Hindi and Bangla
            4: r"[。…‥\.!?？！;\r\n]+(?:\s|$)+",  # Japanese and Chinese
            5: r"[\r\n]+",  # Thai
            6: r"[\r\n؟!\.…]+(?:\s|$)+",
        }  # Arabic and Farsi
        if "sentence_splitter_group" not in self.info:
            split_reg = abbreviation_string + splitters_dict[1]
            sentences = re.split(split_reg, string)
        else:
            split_reg = (
                abbreviation_string
                + splitters_dict[self.info["sentence_splitter_group"]]
            )
            sentences = re.split(split_reg, string)

        sentences = filter(None, sentences)
        return sentences

    def _simplify_split_align(self, original, settings):
        # TODO: Switch to new split method.
        original_tokens = self._word_split(original, settings=settings)
        simplified_tokens = self._word_split(
            self._simplify(normalize_unicode(original), settings=settings),
            settings=settings,
        )
        if len(original_tokens) == len(simplified_tokens):
            return original_tokens, simplified_tokens

        elif len(original_tokens) < len(simplified_tokens):
            add_empty = False
            for i, token in enumerate(simplified_tokens):
                if i < len(original_tokens):
                    if token == normalize_unicode(original_tokens[i].lower()):
                        add_empty = False
                    else:
                        if not add_empty:
                            add_empty = True
                            continue
                        else:
                            original_tokens.insert(i, "")
                else:
                    original_tokens.insert(i, "")
        else:
            add_empty = False
            for i, token in enumerate(original_tokens):
                if i < len(simplified_tokens):
                    if normalize_unicode(token.lower()) == simplified_tokens[i]:
                        add_empty = False
                    else:
                        if not add_empty:
                            add_empty = True
                            continue
                        else:
                            simplified_tokens.insert(i, "")
                else:
                    simplified_tokens.insert(i, "")

        while len(original_tokens) != len(simplified_tokens):
            if len(original_tokens) > len(simplified_tokens):
                original_tokens.remove("")
            else:
                simplified_tokens.remove("")
        return original_tokens, simplified_tokens

    def _get_split_dictionary(self, settings):
        if self._split_dictionary is None:
            settings.NORMALIZE = True
            dictionary = self._get_dictionary(settings=settings)
            self._split_dictionary = self._split_dict(dictionary)
        return self._split_dictionary

    def _split_dict(self, dictionary):
        newdict = {}
        for item in dictionary:
            if " " in item:
                items = item.split()
                for i in items:
                    newdict[i] = dictionary[item]
            else:
                newdict[item] = dictionary[item]
        return newdict

    def _word_split(self, string, settings):
        if "no_word_spacing" in self.info:
            return self._split(string, keep_formatting=True, settings=settings)
        else:
            return string.split()

    def _split(self, date_string, keep_formatting, settings=None):
        tokens = [date_string]
        tokens = list(self._split_tokens_with_regex(tokens, r"(\d+)"))
        tokens = list(
            self._split_tokens_by_known_words(
                tokens, keep_formatting, settings=settings
            )
        )
        return tokens

    def _split_tokens_with_regex(self, tokens, regex):
        tokens = tokens[:]
        for i, token in enumerate(tokens):
            tokens[i] = re.split(regex, token)
        return filter(bool, chain.from_iterable(tokens))

    def _split_tokens_by_known_words(self, tokens, keep_formatting, settings=None):
        dictionary = self._get_dictionary(settings)
        for i, token in enumerate(tokens):
            tokens[i] = dictionary.split(token, keep_formatting)
        return list(chain.from_iterable(tokens))

    def _join_chunk(self, chunk, settings):
        if "no_word_spacing" in self.info:
            return self._join(chunk, separator="", settings=settings)
        else:
            return re.sub(r"\s{2,}", " ", " ".join(chunk))

    def _token_with_digits_is_ok(self, token):
        if "no_word_spacing" in self.info:
            if re.search(r"[\d\.:\-/]+", token) is not None:
                return True
            else:
                return False

        else:
            if re.search(r"\d+", token) is not None:
                return True
            else:
                return False

    def _simplify(self, date_string, settings=None):
        date_string = date_string.lower()
        simplifications = self._get_simplifications(settings=settings)

        if self.info.get("name") == "ru":
            date_string = self._process_russian_compound_ordinals(
                date_string, simplifications
            )
        else:
            date_string = self._apply_simplifications(date_string, simplifications)

        return date_string

    def _apply_simplifications(self, date_string, simplifications):
        for simplification in simplifications:
            pattern, replacement = list(simplification.items())[0]
            date_string = pattern.sub(replacement, date_string).lower()
        return date_string

    def _process_russian_compound_ordinals(self, date_string, simplifications):
        """Process Russian compound ordinals mathematically (двадцать + первое = 21)."""
        date_string = self._apply_simplifications(date_string, simplifications)

        def replace_number_pairs(match):
            first_num = int(match.group(1))
            second_num = int(match.group(2))
            result = first_num + second_num
            if 1 <= result <= 31 and first_num in [20, 30] and 1 <= second_num <= 9:
                return str(result)
            return match.group(0)

        number_pair_pattern = r"\b(\d+)\s+(\d+)\b"
        date_string = re.sub(number_pair_pattern, replace_number_pairs, date_string)

        return date_string

    def _get_simplifications(self, settings=None):
        no_word_spacing = _parse_bool(self.info.get("no_word_spacing", False))
        if settings.NORMALIZE:
            if self._normalized_simplifications is None:
                self._normalized_simplifications = []
                simplifications = self._generate_simplifications(normalize=True)
                for simplification in simplifications:
                    pattern, replacement = list(simplification.items())[0]
                    if not no_word_spacing:
                        pattern = r"(?<=\A|\W|_)%s(?=\Z|\W|_)" % pattern
                    pattern = re.compile(pattern, flags=re.I | re.U)
                    self._normalized_simplifications.append({pattern: replacement})
            return self._normalized_simplifications

        else:
            if self._simplifications is None:
                self._simplifications = []
                simplifications = self._generate_simplifications(normalize=False)
                for simplification in simplifications:
                    pattern, replacement = list(simplification.items())[0]
                    if not no_word_spacing:
                        pattern = r"(?<=\A|\W|_)%s(?=\Z|\W|_)" % pattern
                    pattern = re.compile(pattern, flags=re.I | re.U)
                    self._simplifications.append({pattern: replacement})
            return self._simplifications

    def _generate_simplifications(self, normalize=False):
        simplifications = []
        for simplification in self.info.get("simplifications", []):
            c_simplification = {}
            key, value = list(simplification.items())[0]
            if normalize:
                key = normalize_unicode(key)

            if isinstance(value, int):
                c_simplification[key] = str(value)
            else:
                c_simplification[key] = normalize_unicode(value) if normalize else value

            simplifications.append(c_simplification)
        return simplifications

    def _clear_future_words(self, words):
        freshness_words = {"day", "week", "month", "year", "hour", "minute", "second"}
        if set(words).isdisjoint(freshness_words):
            words.remove("in")
        return words

    def _join(self, tokens, separator=" ", settings=None):
        if not tokens:
            return ""

        capturing_splitters = self._get_splitters(settings)["capturing"]
        joined = tokens[0]
        for i in range(1, len(tokens)):
            left, right = tokens[i - 1], tokens[i]
            if left not in capturing_splitters and right not in capturing_splitters:
                joined += separator
            joined += right

        return joined

    def _get_dictionary(self, settings=None):
        if not settings.NORMALIZE:
            if self._dictionary is None:
                self._generate_dictionary()
            self._dictionary._settings = settings
            return self._dictionary
        else:
            if self._normalized_dictionary is None:
                self._generate_normalized_dictionary()
            self._normalized_dictionary._settings = settings
            return self._normalized_dictionary

    def _get_wordchars(self, settings=None):
        if self._wordchars is None:
            self._set_wordchars(settings)
        return self._wordchars

    def _get_splitters(self, settings=None):
        if self._splitters is None:
            self._set_splitters(settings)
        return self._splitters

    def _set_splitters(self, settings=None):
        splitters = {
            # The ones that split string only if they are not surrounded by letters from both sides:
            "wordchars": set(),
            # The ones that are not filtered out from tokens after split:
            "capturing": set(),
        }
        splitters["capturing"] |= set(ALWAYS_KEEP_TOKENS)

        wordchars = self._get_wordchars(settings)
        skip = set(self.info.get("skip", [])) | splitters["capturing"]
        for token in skip:
            if not re.match(r"^\W+$", token, re.UNICODE):
                continue
            if token in wordchars:
                splitters["wordchars"].add(token)

        self._splitters = splitters

    def _set_wordchars(self, settings=None):
        wordchars = set()
        for word in self._get_dictionary(settings):
            if re.match(r"^[\W\d_]+$", word, re.UNICODE):
                continue
            for char in word:
                wordchars.add(char.lower())

        self._wordchars = wordchars - {" "} | {
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
        }

    def get_wordchars_for_detection(self, settings):
        if self._wordchars_for_detection is None:
            wordchars = set()
            for word in self._get_dictionary(settings):
                if re.match(r"^[\W\d_]+$", word, re.UNICODE):
                    continue
                for char in word:
                    wordchars.add(char.lower())
            self._wordchars_for_detection = wordchars - {
                "0",
                "1",
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
                ":",
                "(",
                ")",
                "'",
                "q",
                "a",
                "m",
                "p",
                " ",
            }
        return self._wordchars_for_detection

    def _generate_dictionary(self, settings=None):
        self._dictionary = Dictionary(self.info, settings=settings)

    def _generate_normalized_dictionary(self, settings=None):
        self._normalized_dictionary = NormalizedDictionary(self.info, settings=settings)

    def to_parserinfo(self, base_cls=parser.parserinfo):
        attributes = {
            "JUMP": self.info.get("skip", []),
            "PERTAIN": self.info.get("pertain", []),
            "WEEKDAYS": [
                self.info["monday"],
                self.info["tuesday"],
                self.info["wednesday"],
                self.info["thursday"],
                self.info["friday"],
                self.info["saturday"],
                self.info["sunday"],
            ],
            "MONTHS": [
                self.info["january"],
                self.info["february"],
                self.info["march"],
                self.info["april"],
                self.info["may"],
                self.info["june"],
                self.info["july"],
                self.info["august"],
                self.info["september"],
                self.info["october"],
                self.info["november"],
                self.info["december"],
            ],
            "HMS": [self.info["hour"], self.info["minute"], self.info["second"]],
        }
        name = "{language}ParserInfo".format(language=self.info["name"])
        return type(name, bases=[base_cls], dict=attributes)



info = {
    "name": "ee",
    "date_order": "MDY",
    "january": [
        "dzove",
        "dzv"
    ],
    "february": [
        "dzd",
        "dzodze"
    ],
    "march": [
        "ted",
        "tedoxe"
    ],
    "april": [
        "afɔ",
        "afɔfĩe"
    ],
    "may": [
        "dam",
        "dama"
    ],
    "june": [
        "mas",
        "masa"
    ],
    "july": [
        "sia",
        "siamlɔm"
    ],
    "august": [
        "dea",
        "deasiamime"
    ],
    "september": [
        "any",
        "anyɔnyɔ"
    ],
    "october": [
        "kel",
        "kele"
    ],
    "november": [
        "ade",
        "adeɛmekpɔxe"
    ],
    "december": [
        "dzm",
        "dzome"
    ],
    "monday": [
        "dzo",
        "dzoɖa"
    ],
    "tuesday": [
        "bla",
        "blaɖa"
    ],
    "wednesday": [
        "kuɖ",
        "kuɖa"
    ],
    "thursday": [
        "yaw",
        "yawoɖa"
    ],
    "friday": [
        "fiɖ",
        "fiɖa"
    ],
    "saturday": [
        "mem",
        "memleɖa"
    ],
    "sunday": [
        "kɔs",
        "kɔsiɖa"
    ],
    "am": [
        "ŋdi"
    ],
    "pm": [
        "ɣetrɔ"
    ],
    "year": [
        "ƒe"
    ],
    "month": [
        "ɣleti"
    ],
    "week": [
        "kɔsiɖa ɖeka"
    ],
    "day": [
        "ŋkeke"
    ],
    "hour": [
        "gaƒoƒo"
    ],
    "minute": [
        "aɖabaƒoƒo"
    ],
    "second": [
        "sekend"
    ],
    "relative-type": {
        "0 day ago": [
            "egbe"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "ɣleti sia"
        ],
        "0 second ago": [
            "fifi"
        ],
        "0 week ago": [
            "kɔsiɖa sia"
        ],
        "0 year ago": [
            "ƒe sia"
        ],
        "1 day ago": [
            "etsɔ si va yi"
        ],
        "1 month ago": [
            "ɣleti si va yi"
        ],
        "1 week ago": [
            "kɔsiɖa si va yi"
        ],
        "1 year ago": [
            "ƒe si va yi"
        ],
        "in 1 day": [
            "etsɔ si gbɔna"
        ],
        "in 1 month": [
            "ɣleti si gbɔ na"
        ],
        "in 1 week": [
            "kɔsiɖa si gbɔ na"
        ],
        "in 1 year": [
            "ƒe si gbɔ na"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "ŋkeke (\\d+[.,]?\\d*) si va yi",
            "ŋkeke (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 hour ago": [
            "gaƒoƒo (\\d+[.,]?\\d*) si va yi",
            "gaƒoƒo (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 minute ago": [
            "aɖabaƒoƒo (\\d+[.,]?\\d*) si va yi",
            "aɖabaƒoƒo (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 month ago": [
            "ɣleti (\\d+[.,]?\\d*) si va yi",
            "ɣleti (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 second ago": [
            "sekend (\\d+[.,]?\\d*) si va yi",
            "sekend (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 week ago": [
            "kɔsiɖa (\\d+[.,]?\\d*) si va yi",
            "kɔsiɖa (\\d+[.,]?\\d*) si wo va yi"
        ],
        "\\1 year ago": [
            "le ƒe (\\d+[.,]?\\d*) si va yi me",
            "ƒe (\\d+[.,]?\\d*) si va yi",
            "ƒe (\\d+[.,]?\\d*) si va yi me",
            "ƒe (\\d+[.,]?\\d*) si wo va yi"
        ],
        "in \\1 day": [
            "le ŋkeke (\\d+[.,]?\\d*) me",
            "le ŋkeke (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 hour": [
            "le gaƒoƒo (\\d+[.,]?\\d*) me",
            "le gaƒoƒo (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 minute": [
            "le aɖabaƒoƒo (\\d+[.,]?\\d*) me",
            "le aɖabaƒoƒo (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 month": [
            "le ɣleti (\\d+[.,]?\\d*) me",
            "le ɣleti (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 second": [
            "le sekend (\\d+[.,]?\\d*) me",
            "le sekend (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 week": [
            "le kɔsiɖa (\\d+[.,]?\\d*) me",
            "le kɔsiɖa (\\d+[.,]?\\d*) wo me"
        ],
        "in \\1 year": [
            "le ƒe (\\d+[.,]?\\d*) me",
            "le ƒe (\\d+[.,]?\\d*) si gbɔna me"
        ]
    },
    "locale_specific": {
        "ee-TG": {
            "name": "ee-TG"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ti",
    "date_order": "DMY",
    "january": [
        "ጥሪ"
    ],
    "february": [
        "ለካ",
        "ለካቲት"
    ],
    "march": [
        "መጋ",
        "መጋቢት"
    ],
    "april": [
        "ሚያ",
        "ሚያዝያ"
    ],
    "may": [
        "ግን",
        "ግንቦት"
    ],
    "june": [
        "ሰነ"
    ],
    "july": [
        "ሓም",
        "ሓምለ"
    ],
    "august": [
        "ነሓ",
        "ነሓሰ"
    ],
    "september": [
        "መስ",
        "መስከረም"
    ],
    "october": [
        "ጥቅ",
        "ጥቅምቲ"
    ],
    "november": [
        "ሕዳ",
        "ሕዳር"
    ],
    "december": [
        "ታሕ",
        "ታሕሳስ"
    ],
    "monday": [
        "ሰኑ",
        "ሰኑይ"
    ],
    "tuesday": [
        "ሠሉስ",
        "ሰሉ",
        "ሰሉስ"
    ],
    "wednesday": [
        "ረቡ",
        "ረቡዕ"
    ],
    "thursday": [
        "ሓሙ",
        "ሓሙስ",
        "ኃሙስ"
    ],
    "friday": [
        "ዓር",
        "ዓርቢ"
    ],
    "saturday": [
        "ቀዳ",
        "ቀዳም"
    ],
    "sunday": [
        "ሰን",
        "ሰንበት"
    ],
    "am": [
        "ንጉሆ ሰዓተ"
    ],
    "pm": [
        "ድሕር ሰዓት"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "ti-ER": {
            "name": "ti-ER"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



"""
Utilities for handling time spans and date ranges.
"""

import re
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta


def get_week_start(date, start_of_week="monday"):
    """Get the start of the week for a given date."""
    if start_of_week == "monday":
        days_back = date.weekday()
    else:  # sunday
        days_back = (date.weekday() + 1) % 7

    return date - timedelta(days=days_back)


def get_week_end(date, start_of_week="monday"):
    """Get the end of the week for a given date."""
    week_start = get_week_start(date, start_of_week)
    return week_start + timedelta(days=6)


def detect_time_span(text):
    """Detect time span expressions in text and return span information."""
    span_patterns = [
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:past|last|previous)\s+month\b",
            "type": "month",
            "direction": "past",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:past|last|previous)\s+week\b",
            "type": "week",
            "direction": "past",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:past|last|previous)\s+(\d+)\s+days?\b",
            "type": "days",
            "direction": "past",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:past|last|previous)\s+(\d+)\s+weeks?\b",
            "type": "weeks",
            "direction": "past",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:past|last|previous)\s+(\d+)\s+months?\b",
            "type": "months",
            "direction": "past",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:next|coming|following)\s+month\b",
            "type": "month",
            "direction": "future",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:next|coming|following)\s+week\b",
            "type": "week",
            "direction": "future",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:next|coming|following)\s+(\d+)\s+days?\b",
            "type": "days",
            "direction": "future",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:next|coming|following)\s+(\d+)\s+weeks?\b",
            "type": "weeks",
            "direction": "future",
        },
        {
            "pattern": r"\b(?:for\s+the\s+|during\s+the\s+|in\s+the\s+)?(?:next|coming|following)\s+(\d+)\s+months?\b",
            "type": "months",
            "direction": "future",
        },
    ]

    for pattern_info in span_patterns:
        match = re.search(pattern_info["pattern"], text, re.IGNORECASE)
        if match:
            result = {
                "type": pattern_info["type"],
                "direction": pattern_info["direction"],
                "matched_text": match.group(0),
                "start_pos": match.start(),
                "end_pos": match.end(),
            }

            if match.groups():
                result["number"] = int(match.group(1))

            return result

    return None


def generate_time_span(span_info, base_date=None, settings=None):
    """Generate start and end dates for a time span."""
    if base_date is None:
        base_date = datetime.now()

    if settings is None:
        start_of_week = "monday"
        days_in_month = 30
    else:
        start_of_week = getattr(settings, "DEFAULT_START_OF_WEEK", "monday")
        days_in_month = getattr(settings, "DEFAULT_DAYS_IN_MONTH", 30)

    span_type = span_info["type"]
    direction = span_info["direction"]
    number = span_info.get("number", 1)

    if direction == "past":
        end_date = base_date

        if span_type == "month":
            start_date = end_date - relativedelta(days=days_in_month)
        elif span_type == "week":
            week_start = get_week_start(end_date, start_of_week)
            start_date = week_start - timedelta(days=7)
            end_date = week_start - timedelta(days=1)
        elif span_type == "days":
            start_date = end_date - timedelta(days=number)
        elif span_type == "weeks":
            start_date = end_date - timedelta(weeks=number)
        elif span_type == "months":
            start_date = end_date - relativedelta(months=number)
        else:
            start_date = end_date - timedelta(days=1)

    else:
        start_date = base_date

        if span_type == "month":
            end_date = start_date + relativedelta(days=days_in_month)
        elif span_type == "week":
            week_start = get_week_start(start_date, start_of_week)
            start_date = week_start + timedelta(days=7)
            end_date = start_date + timedelta(days=6)
        elif span_type == "days":
            end_date = start_date + timedelta(days=number)
        elif span_type == "weeks":
            end_date = start_date + timedelta(weeks=number)
        elif span_type == "months":
            end_date = start_date + relativedelta(months=number)
        else:
            end_date = start_date + timedelta(days=1)

    return (start_date, end_date)



info = {
    "name": "sw",
    "date_order": "DMY",
    "january": [
        "jan",
        "januari"
    ],
    "february": [
        "feb",
        "februari"
    ],
    "march": [
        "mac",
        "machi"
    ],
    "april": [
        "apr",
        "aprili"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "juni"
    ],
    "july": [
        "jul",
        "julai"
    ],
    "august": [
        "ago",
        "agosti"
    ],
    "september": [
        "sep",
        "septemba"
    ],
    "october": [
        "okt",
        "oktoba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "des",
        "desemba"
    ],
    "monday": [
        "jumatatu"
    ],
    "tuesday": [
        "jumanne"
    ],
    "wednesday": [
        "jumatano"
    ],
    "thursday": [
        "alhamisi"
    ],
    "friday": [
        "ijumaa"
    ],
    "saturday": [
        "jumamosi"
    ],
    "sunday": [
        "jumapili"
    ],
    "am": [
        "am",
        "asubuhi"
    ],
    "pm": [
        "mchana",
        "pm"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mwezi"
    ],
    "week": [
        "wiki"
    ],
    "day": [
        "siku"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "dak",
        "dakika"
    ],
    "second": [
        "sek",
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "leo"
        ],
        "0 hour ago": [
            "saa hii"
        ],
        "0 minute ago": [
            "dakika hii"
        ],
        "0 month ago": [
            "mwezi huu"
        ],
        "0 second ago": [
            "sasa hivi"
        ],
        "0 week ago": [
            "wiki hii"
        ],
        "0 year ago": [
            "mwaka huu"
        ],
        "1 day ago": [
            "jana"
        ],
        "1 month ago": [
            "mwezi uliopita"
        ],
        "1 week ago": [
            "wiki iliyopita"
        ],
        "1 year ago": [
            "mwaka uliopita"
        ],
        "in 1 day": [
            "kesho"
        ],
        "in 1 month": [
            "mwezi ujao"
        ],
        "in 1 week": [
            "wiki ijayo"
        ],
        "in 1 year": [
            "mwaka ujao"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "siku (\\d+[.,]?\\d*) iliyopita",
            "siku (\\d+[.,]?\\d*) zilizopita"
        ],
        "\\1 hour ago": [
            "saa (\\d+[.,]?\\d*) iliyopita",
            "saa (\\d+[.,]?\\d*) zilizopita"
        ],
        "\\1 minute ago": [
            "dakika (\\d+[.,]?\\d*) iliyopita",
            "dakika (\\d+[.,]?\\d*) zilizopita"
        ],
        "\\1 month ago": [
            "miezi (\\d+[.,]?\\d*) iliyopita",
            "mwezi (\\d+[.,]?\\d*) uliopita"
        ],
        "\\1 second ago": [
            "sekunde (\\d+[.,]?\\d*) iliyopita",
            "sekunde (\\d+[.,]?\\d*) zilizopita"
        ],
        "\\1 week ago": [
            "wiki (\\d+[.,]?\\d*) iliyopita",
            "wiki (\\d+[.,]?\\d*) zilizopita"
        ],
        "\\1 year ago": [
            "miaka (\\d+[.,]?\\d*) iliyopita",
            "mwaka (\\d+[.,]?\\d*) uliopita"
        ],
        "in \\1 day": [
            "baada ya siku (\\d+[.,]?\\d*)"
        ],
        "in \\1 hour": [
            "baada ya saa (\\d+[.,]?\\d*)"
        ],
        "in \\1 minute": [
            "baada ya dakika (\\d+[.,]?\\d*)"
        ],
        "in \\1 month": [
            "baada ya miezi (\\d+[.,]?\\d*)",
            "baada ya mwezi (\\d+[.,]?\\d*)"
        ],
        "in \\1 second": [
            "baada ya sekunde (\\d+[.,]?\\d*)"
        ],
        "in \\1 week": [
            "baada ya wiki (\\d+[.,]?\\d*)"
        ],
        "in \\1 year": [
            "baada ya miaka (\\d+[.,]?\\d*)",
            "baada ya mwaka (\\d+[.,]?\\d*)"
        ]
    },
    "locale_specific": {
        "sw-CD": {
            "name": "sw-CD",
            "week": [
                "juma"
            ]
        },
        "sw-KE": {
            "name": "sw-KE"
        },
        "sw-UG": {
            "name": "sw-UG"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "dje",
    "date_order": "DMY",
    "january": [
        "žan",
        "žanwiye"
    ],
    "february": [
        "fee",
        "feewiriye"
    ],
    "march": [
        "mar",
        "marsi"
    ],
    "april": [
        "awi",
        "awiril"
    ],
    "may": [
        "me"
    ],
    "june": [
        "žuw",
        "žuweŋ"
    ],
    "july": [
        "žuy",
        "žuyye"
    ],
    "august": [
        "ut"
    ],
    "september": [
        "sek",
        "sektanbur"
    ],
    "october": [
        "okt",
        "oktoobur"
    ],
    "november": [
        "noo",
        "noowanbur"
    ],
    "december": [
        "dee",
        "deesanbur"
    ],
    "monday": [
        "ati",
        "atinni"
    ],
    "tuesday": [
        "ata",
        "atalaata"
    ],
    "wednesday": [
        "ala",
        "alarba"
    ],
    "thursday": [
        "alhamisi",
        "alm"
    ],
    "friday": [
        "alz",
        "alzuma"
    ],
    "saturday": [
        "asi",
        "asibti"
    ],
    "sunday": [
        "alh",
        "alhadi"
    ],
    "am": [
        "subbaahi"
    ],
    "pm": [
        "zaarikay b"
    ],
    "year": [
        "jiiri"
    ],
    "month": [
        "handu"
    ],
    "week": [
        "hebu"
    ],
    "day": [
        "zaari"
    ],
    "hour": [
        "guuru"
    ],
    "minute": [
        "miniti"
    ],
    "second": [
        "miti"
    ],
    "relative-type": {
        "0 day ago": [
            "hõo"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "bi"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "suba"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ig",
    "date_order": "DMY",
    "january": [
        "jen",
        "jenụwarị"
    ],
    "february": [
        "feb",
        "febrụwarị"
    ],
    "march": [
        "maa",
        "maachị"
    ],
    "april": [
        "epr",
        "eprel"
    ],
    "may": [
        "mee"
    ],
    "june": [
        "juu",
        "juun"
    ],
    "july": [
        "jul",
        "julaị"
    ],
    "august": [
        "ọgọ",
        "ọgọọst"
    ],
    "september": [
        "sep",
        "septemba"
    ],
    "october": [
        "ọkt",
        "ọktoba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "dis",
        "disemba"
    ],
    "monday": [
        "mọn",
        "mọnde"
    ],
    "tuesday": [
        "tiu",
        "tiuzdee"
    ],
    "wednesday": [
        "wen",
        "wenezdee"
    ],
    "thursday": [
        "tọọ",
        "tọọzdee"
    ],
    "friday": [
        "fraị",
        "fraịdee"
    ],
    "saturday": [
        "sat",
        "satọdee"
    ],
    "sunday": [
        "mbọsị ụka",
        "ụka"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "afọ"
    ],
    "month": [
        "ọnwa"
    ],
    "week": [
        "izu"
    ],
    "day": [
        "ụbọchị"
    ],
    "hour": [
        "elekere"
    ],
    "minute": [
        "nkeji"
    ],
    "second": [
        "nkejinta"
    ],
    "relative-type": {
        "0 day ago": [
            "taata"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "nnyaafụ"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "echi"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import json
import os
import shutil
from collections import OrderedDict

import regex as re
from ruamel.yaml import YAML

from dateparser_scripts.order_languages import avoid_languages
from dateparser_scripts.utils import combine_dicts

cldr_date_directory = "../dateparser_data/cldr_language_data/date_translation_data/"
supplementary_directory = "../dateparser_data/supplementary_language_data/"
supplementary_date_directory = (
    "../dateparser_data/supplementary_language_data/date_translation_data/"
)
translation_data_directory = "../dateparser/data/"
date_translation_directory = "../dateparser/data/date_translation_data/"

os.chdir(os.path.dirname(os.path.abspath(__file__)))

cldr_languages = list(
    set(map(lambda x: x[:-5], os.listdir(cldr_date_directory))) - avoid_languages
)
supplementary_languages = [x[:-5] for x in os.listdir(supplementary_date_directory)]
all_languages = set(cldr_languages).union(set(supplementary_languages))

RELATIVE_PATTERN = re.compile(r"\{0\}")


def _modify_relative_data(relative_data):
    modified_relative_data = OrderedDict()
    for key, value in relative_data.items():
        for i, string in enumerate(value):
            string = RELATIVE_PATTERN.sub(r"(\\d+[.,]?\\d*)", string)
            value[i] = string
        modified_relative_data[key] = value
    return modified_relative_data


def _modify_data(language_data):
    relative_data = language_data.get("relative-type-regex", {})
    relative_data = _modify_relative_data(relative_data)
    locale_specific_data = language_data.get("locale_specific", {})
    for _, info in locale_specific_data.items():
        locale_relative_data = info.get("relative-type-regex", {})
        locale_relative_data = _modify_relative_data(locale_relative_data)


def _get_complete_date_translation_data(language):
    cldr_data = {}
    supplementary_data = {}
    if language in cldr_languages:
        with open(cldr_date_directory + language + ".json") as f:
            cldr_data = json.load(f, object_pairs_hook=OrderedDict)
    if language in supplementary_languages:
        with open(supplementary_date_directory + language + ".yaml") as g:
            yaml = YAML()
            supplementary_data = OrderedDict(yaml.load(g))
    complete_data = combine_dicts(cldr_data, supplementary_data)
    if "name" not in complete_data:
        complete_data["name"] = language
    return complete_data


def _write_file(filename, text, mode, in_memory, in_memory_result):
    if in_memory:
        in_memory_result[filename] = text
    else:
        with open(filename, mode) as out:
            out.write(text)


def write_complete_data(in_memory=False):
    """
    This function is responsible of generating the needed py files from the
    CLDR files (JSON format) and supplementary language data (YAML format).

    Use it with in_memory=True to avoid writing real files and getting a
    dictionary containing the file names and their content (used when testing).
    """
    in_memory_result = {}

    if not in_memory:
        if not os.path.isdir(translation_data_directory):
            os.mkdir(translation_data_directory)
        if os.path.isdir(date_translation_directory):
            shutil.rmtree(date_translation_directory)
        os.mkdir(date_translation_directory)

    with open(supplementary_directory + "base_data.yaml") as f:
        yaml = YAML()
        base_data = yaml.load(f)

    for language in all_languages:
        date_translation_data = _get_complete_date_translation_data(language)
        date_translation_data = combine_dicts(date_translation_data, base_data)
        _modify_data(date_translation_data)
        translation_data = json.dumps(
            date_translation_data, indent=4, separators=(",", ": "), ensure_ascii=False
        )
        out_text = ("info = " + translation_data + "\n").encode("utf-8")
        _write_file(
            date_translation_directory + language + ".py",
            out_text,
            "wb",
            in_memory,
            in_memory_result,
        )

    init_text = (
        "from dateparser.data import date_translation_data\n"
        "from .languages_info import language_order, language_locale_dict\n"
    )

    _write_file(
        translation_data_directory + "__init__.py",
        init_text,
        "w",
        False,
        in_memory_result,
    )
    _write_file(
        date_translation_directory + "__init__.py", "", "w", False, in_memory_result
    )

    return in_memory_result


if __name__ == "__main__":
    write_complete_data()



info = {
    "name": "agq",
    "date_order": "DMY",
    "january": [
        "ndzɔ̀ŋɔ̀nùm",
        "nùm"
    ],
    "february": [
        "kɨz",
        "ndzɔ̀ŋɔ̀kɨ̀zùʔ"
    ],
    "march": [
        "ndzɔ̀ŋɔ̀tɨ̀dʉ̀ghà",
        "tɨd"
    ],
    "april": [
        "ndzɔ̀ŋɔ̀tǎafʉ̄ghā",
        "taa"
    ],
    "may": [
        "ndzɔ̀ŋèsèe",
        "see"
    ],
    "june": [
        "ndzɔ̀ŋɔ̀nzùghò",
        "nzu"
    ],
    "july": [
        "dum",
        "ndzɔ̀ŋɔ̀dùmlo"
    ],
    "august": [
        "fɔe",
        "ndzɔ̀ŋɔ̀kwîfɔ̀e"
    ],
    "september": [
        "dzu",
        "ndzɔ̀ŋɔ̀tɨ̀fʉ̀ghàdzughù"
    ],
    "october": [
        "lɔm",
        "ndzɔ̀ŋɔ̀ghǔuwelɔ̀m"
    ],
    "november": [
        "kaa",
        "ndzɔ̀ŋɔ̀chwaʔàkaa wo"
    ],
    "december": [
        "fwo",
        "ndzɔ̀ŋèfwòo"
    ],
    "monday": [
        "kpa",
        "tsuʔukpà"
    ],
    "tuesday": [
        "ghɔ",
        "tsuʔughɔe"
    ],
    "wednesday": [
        "tsuʔutɔ̀mlò",
        "tɔm"
    ],
    "thursday": [
        "tsuʔumè",
        "ume"
    ],
    "friday": [
        "ghɨ",
        "tsuʔughɨ̂m"
    ],
    "saturday": [
        "dzk",
        "tsuʔndzɨkɔʔɔ"
    ],
    "sunday": [
        "nts",
        "tsuʔntsɨ"
    ],
    "am": [
        "ag"
    ],
    "pm": [
        "ak"
    ],
    "year": [
        "kɨnûm"
    ],
    "month": [
        "ndzɔŋ"
    ],
    "week": [
        "ewɨn"
    ],
    "day": [
        "utsuʔ"
    ],
    "hour": [
        "tàm"
    ],
    "minute": [
        "menè"
    ],
    "second": [
        "sɛkɔ̀n"
    ],
    "relative-type": {
        "0 day ago": [
            "nɛ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ā zūɛɛ"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tsʉtsʉ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}
